using System;
using dk.esthauge.limid.gui;
namespace dk.esthauge.limid
{
	
	/// <summary> A class containing a main method activating the LIMID GUI.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class Limid
	{
		internal bool packFrame = true;
		
		//Construct the application
		public Limid()
		{
			LimidFrame frame = new LimidFrame();
			//Validate frames that have preset sizes
			//Pack frames that have useful preferred size info, e.g. from their layout
			if (packFrame)
			{
				//UPGRADE_ISSUE: Method 'java.awt.Window.pack' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1000_javaawtWindowpack"'
				frame.pack();
			}
			else
			{
				frame.Validate();
			}
			//Center the window
			//UPGRADE_ISSUE: Method 'java.awt.Toolkit.getDefaultToolkit' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1000_javaawtToolkit"'
			Toolkit.getDefaultToolkit();
			System.Drawing.Size screenSize = System.Windows.Forms.Screen.PrimaryScreen.Bounds.Size;
			System.Drawing.Size frameSize = frame.Size;
			if (frameSize.Height > screenSize.Height)
			{
				frameSize.Height = screenSize.Height;
			}
			if (frameSize.Width > screenSize.Width)
			{
				frameSize.Width = screenSize.Width;
			}
			//UPGRADE_TODO: Method 'java.awt.Component.setLocation' was converted to 'System.Windows.Forms.Control.Location' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073"'
			frame.Location = new System.Drawing.Point((screenSize.Width - frameSize.Width) / 2, (screenSize.Height - frameSize.Height) / 2);
			//UPGRADE_TODO: Method 'java.awt.Component.setVisible' was converted to 'System.Windows.Forms.Control.Visible' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073"'
			//UPGRADE_TODO: System.Windows.Forms.Application.Run must be called to start a main form. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1135"'
			frame.Visible = true;
		}
		/// <summary> Activates the LIMID GUI. If no parameter is given, the GUI is activated
		/// with the Metal look-and-feel.
		/// </summary>
		/// <param name="args">If the string "native" is given as parameter, the GUI will use
		/// the native look-and-feel of the operating system.
		/// </param>
		[STAThread]
		public static void  Main(System.String[] args)
		{
			if (args.Length > 0 && args[0].ToUpper().Equals("native".ToUpper()))
				try
				{
					//UPGRADE_TODO: Method 'javax.swing.UIManager.setLookAndFeel' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					//UPGRADE_TODO: Method 'javax.swing.UIManager.getSystemLookAndFeelClassName' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
				}
				catch (System.Exception e)
				{
					SupportClass.WriteStackTrace(e, Console.Error);
				}
			new Limid();
		}
	}
}