using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> A chance node with two states. The default state labels are "false" and "true".
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// <p>Company: </p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class BooleanLimidChanceNode:LimidChanceNode, BooleanNode
	{
		
		/// <summary> Creates a new boolean chance node within the specified domain.</summary>
		/// <param name="ld">The domain.
		/// @throws MLPDuplicateNodeNameException
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public BooleanLimidChanceNode(LimidDomain ld):base(ld)
		{
			base.setNumberOfStates(2);
			this.setStateLabel(0, "false");
			this.setStateLabel(1, "true");
		}
		
		/// <summary> Since a boolean node has always 2 states, this method has no effect at all.
		/// It is only included as a result of inheritance.
		/// </summary>
		/// <param name="i">The specified number of states. The value is ignored.
		/// </param>
		public override void  setNumberOfStates(int i)
		{
		}
		
		
		/// <summary> Returns the string "boolean".</summary>
		/// <returns> The string "boolean".
		/// </returns>
		protected internal override System.String getSubtype()
		{
			return "boolean";
		}
		
		//UPGRADE_TODO: The equivalent of method 'java.lang.Object.clone' is not an override method. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1143"'
		protected internal System.Object Clone()
		{
			try
			{
				BooleanLimidChanceNode newNode = new BooleanLimidChanceNode(this.HomeDomain);
				newNode.copyNode(this);
				return newNode;
			}
			catch (MLPException ex)
			{
				return null;
			}
		}
	}
}