using System;
using dk.kvl.ihh.prodstyr.mlhmp;
namespace dk.esthauge.limid.api
{
	
	/// <summary> The default probability table of all discrete nodes.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class DefaultProbabilityPotential:AbstractPotential
	{
		override public double[] Data
		{
			get
			{
				double[] data = new double[size];
				for (int index = 0; index < size; index++)
				{
					int i = index / stateCount;
					int j = index % stateCount;
					data[index] = potentials[i].get_Renamed(j);
				}
				return data;
			}
			
			set
			{
				for (int index = 0; index < size; index++)
				{
					int i = index / stateCount;
					int j = index % stateCount;
					//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
					potentials[i].set_Renamed(j, (float) value[index]);
				}
			}
			
		}
		
		/// <summary> An array of probability sets. One for each parent configuration.</summary>
		private ProbabilityProvider[] potentials;
		
		/// <summary> Creates an empty probability table based on the nodes defining the state
		/// space. The ProbabilityProvider type used as standard is an ArrayProbabilityProvider
		/// </summary>
		/// <param name="nodes">The nodes
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public DefaultProbabilityPotential(NodeList nodes):base(nodes)
		{
			potentials = new ProbabilityProvider[parentStateCount];
			for (int j = 0; j < potentials.length; j++)
				potentials[j] = new ArrayProbabilityProvider(stateCount);
		}
		
		public override double getDataItem(int index)
		{
			int pi = index / stateCount;
			int ti = index % stateCount;
			return potentials[pi].get_Renamed(ti);
		}
		
		public override double getDataItem(int[] state)
		{
			int pi = this.convertToParentIndex(state);
			int ti = state[state.Length - 1];
			return potentials[pi].get_Renamed(ti);
		}
		
		public override void  setDataItem(int index, double value_Renamed)
		{
			int pi = index / stateCount;
			int ti = index % stateCount;
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			potentials[pi].set_Renamed(ti, (float) value_Renamed);
		}
		
		public override void  setDataItem(int[] state, double value_Renamed)
		{
			int pi = this.convertToParentIndex(state);
			int ti = state[state.Length - 1];
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			potentials[pi].set_Renamed(ti, (float) value_Renamed);
		}
		
		/// <summary> Sets the ProbabilityProvider for a specified parent state.</summary>
		/// <param name="parentIndex">The index of the parent configuration (parent state)
		/// </param>
		/// <param name="pp">The desired probability provider
		/// </param>
		public virtual void  setProbabilityProvider(int parentIndex, ProbabilityProvider pp)
		{
			potentials[parentIndex] = pp;
		}
		
		/// <summary> Returns the probability provider for a specified parent state.</summary>
		/// <param name="parentIndex">The index of the parent configuration (parent state)
		/// </param>
		/// <returns> The probability provider.
		/// </returns>
		public virtual ProbabilityProvider getProbabilityProvider(int parentIndex)
		{
			return potentials[parentIndex];
		}
	}
}