using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> A directed edge between two nodes of the network.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class DirectedEdge:Edge
	{
		/// <summary> Calculate the arc corners given a specified arc length. The method is
		/// called by the API. It should not be called directly.
		/// </summary>
		/// <param name="arcLength">The arc length
		/// </param>
		virtual protected internal double ArcCorners
		{
			set
			{
				double slope = Slope;
				double arcEndLength = 0.33333 * value;
				//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				Point2D arcEndCenter = this.findNewTargetOffset(value);
				double a1X, a1Y, a2X, a2Y;
				if (slope != 0)
				{
					double endSlope = (- 1) / Slope;
					double xdif = System.Math.Sqrt(arcEndLength * arcEndLength / (1 + endSlope * endSlope));
					double ydif = xdif * endSlope;
					if (endSlope > 0)
					{
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						a1X = arcEndCenter.getX() + xdif + 1;
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						a1Y = arcEndCenter.getY() + ydif + 1;
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						a2X = arcEndCenter.getX() - xdif;
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						a2Y = arcEndCenter.getY() - ydif;
					}
					else
					{
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						a1X = arcEndCenter.getX() + xdif + 1;
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						a1Y = arcEndCenter.getY() + ydif;
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						a2X = arcEndCenter.getX() - xdif;
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						a2Y = arcEndCenter.getY() - ydif + 1;
					}
				}
				else
				{
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					a1X = arcEndCenter.getX();
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					a1Y = arcEndCenter.getY() - arcEndLength;
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					a2X = arcEndCenter.getX();
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					a2Y = arcEndCenter.getY() + arcEndLength + 1;
				}
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				arc1 = new System.Drawing.Point((int) a1X, (int) a1Y);
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				arc2 = new System.Drawing.Point((int) a2X, (int) a2Y);
			}
			
		}
		/// <summary> One of the corners of the arc</summary>
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		private Point2D arc1;
		
		/// <summary> The other corner of the arc</summary>
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		private Point2D arc2;
		
		/// <summary> Creates an edge between two nodes</summary>
		/// <param name="parent">The neighbor (parent)
		/// </param>
		/// <param name="child">The owner (child
		/// </param>
		public DirectedEdge(Node parent, Node child):this(parent, child, 0.0)
		{
		}
		
		/// <summary> Creates an edge between two nodes. The edge is shortened in both ends by
		/// the value specified.
		/// </summary>
		/// <param name="parent">The neighbor (parent)
		/// </param>
		/// <param name="child">The owner (child
		/// </param>
		/// <param name="abbrevation">The value used for shortening the edge
		/// </param>
		public DirectedEdge(Node parent, Node child, double abbrevation):base(parent, child, abbrevation)
		{
			this.ArcCorners = abbrevation / 2;
		}
		
		public override void  draw(System.Drawing.Graphics graphics)
		{
			if (!drawSelected_Renamed_Field)
				SupportClass.GraphicsManager.manager.SetColor(graphics, this.Color);
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			graphics.DrawLine(SupportClass.GraphicsManager.manager.GetPen(graphics), (int) origin.getX(), (int) origin.getY(), (int) target.getX(), (int) target.getY());
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			int[] x = new int[]{(int) arc1.getX(), (int) arc2.getX(), (int) target.getX()};
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			int[] y = new int[]{(int) arc1.getY(), (int) arc2.getY(), (int) target.getY()};
			graphics.FillPath(SupportClass.GraphicsManager.manager.GetBrush(graphics), SupportClass.CreateGraphicsPath(x, y, 3));
			SupportClass.GraphicsManager.manager.SetColor(graphics, System.Drawing.Color.Black);
		}
	}
}