using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> An interface specifying the methods used for drawing items on a graphics
	/// context.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// <p>Company: </p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public interface Drawable
		{
			
			/// <summary> Draws this item on a specified graphics context.</summary>
			/// <param name="graphics">The graphics context.
			/// </param>
			void  draw(System.Drawing.Graphics graphics);
			
			/// <summary> Draws this item as selected on a specified graphics context.</summary>
			/// <param name="graphics">The graphics context.
			/// </param>
			void  drawSelected(System.Drawing.Graphics graphics);
			
			/// <summary> Checks whether the drawable item contains the specified point. Used to
			/// determine whether an item is selected by a mouse click.
			/// </summary>
			/// <param name="point">The point
			/// </param>
			/// <returns> true if the point is within this item.
			/// </returns>
			//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			bool isPointWithinShape(Point2D point);
			
			/// <summary> Returns the item if it is selected. In implementing classes the method
			/// uses isPointWithinShape(point) to determine whether the item is selected
			/// or not.
			/// </summary>
			/// <param name="point">A point that defines the selection criterion.
			/// </param>
			/// <returns> The item that is selected (null if nothing is selected).
			/// </returns>
			//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			Drawable getSelectedItem(Point2D point);
			
			/// <summary> Adds this item to a list of already selected items if it is situated
			/// within the rectangle.
			/// </summary>
			/// <param name="r">The rectangle defining the selection area.
			/// </param>
			/// <param name="allready">The list of already selected items.
			/// </param>
			/// <returns> The updated list of selected items.
			/// </returns>
			//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			SupportClass.CollectionSupport addSelectedItems(System.Drawing.Rectangle r, SupportClass.CollectionSupport allready);
		}
}