using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> An edge between two nodes.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class Edge : Drawable
	{
		/// <summary> Returns the slope of the edge</summary>
		/// <returns> The slope
		/// </returns>
		virtual public double Slope
		{
			get
			{
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				double x = origin.getX() - target.getX();
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				double y = origin.getY() - target.getY();
				double slope;
				if (x != 0)
					slope = y / x;
				else
					slope = 1.0e300;
				return slope;
			}
			
		}
		/// <summary> Returns the color of the edge.</summary>
		/// <returns> The color
		/// </returns>
		virtual public System.Drawing.Color Color
		{
			get
			{
				return color;
			}
			
		}
		/// <summary> Returns the intercept of the edge</summary>
		/// <returns> The intercept
		/// </returns>
		virtual public double Intercept
		{
			get
			{
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				return (origin.getY() - Slope * origin.getX());
			}
			
		}
		/// <summary> Returns the owner.</summary>
		/// <returns> The owner.
		/// </returns>
		virtual public Node Owner
		{
			get
			{
				return owner;
			}
			
		}
		/// <summary> Returns the neighbor.</summary>
		/// <returns> The neighbor.
		/// </returns>
		virtual public Node Neighbor
		{
			get
			{
				return neighbor;
			}
			
		}
		/// <summary> Sets the sensitivity.</summary>
		/// <param name="sensitivity">The desired sensitivity
		/// </param>
		/// <seealso cref="Edge#sens">
		/// </seealso>
		public static int ClickDistanceSensitivity
		{
			set
			{
				sens = value;
			}
			
		}
		/// <summary> The sensitivity concerning selection by mouse click. The integer value
		/// specifies the maximum distance from the edge that results in selection.
		/// </summary>
		internal static int sens = 4;
		
		/// <summary> The offset of the edge</summary>
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		protected internal Point2D origin;
		
		/// <summary> The end point of the edge</summary>
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		protected internal Point2D target;
		
		/// <summary> The node that owns this edge</summary>
		protected internal Node owner;
		
		/// <summary> The node that the owner is connected to by this edge</summary>
		protected internal Node neighbor;
		
		/// <summary> The color of this edge</summary>
		protected internal System.Drawing.Color color;
		
		/// <summary> Should this edge be drawn as selected.</summary>
		protected internal bool drawSelected_Renamed_Field = false;
		
		/// <summary> Creates a black edge between two nodes</summary>
		/// <param name="parent">The neigbor.
		/// </param>
		/// <param name="child">The node that owns the edge.
		/// </param>
		public Edge(Node parent, Node child):this(parent, child, 0.0)
		{
		}
		
		/// <summary> Creates a black edge between two nodes. The edge is shortened in both ends by
		/// the value specified.
		/// </summary>
		/// <param name="parent">The neighbor
		/// </param>
		/// <param name="child">The node that owns the edge.
		/// </param>
		/// <param name="abbrevation">The value used for shortening the edge
		/// </param>
		public Edge(Node parent, Node child, double abbrevation):this(parent, child, abbrevation, System.Drawing.Color.Black)
		{
		}
		
		/// <summary> Creates an edge of a specified color between two nodes. The edge is
		/// shortened in both ends by the value specified.
		/// </summary>
		/// <param name="parent">The neighbor
		/// </param>
		/// <param name="child">The node that owns the edge.
		/// </param>
		/// <param name="abbrevation">The value used for shortening the edge
		/// </param>
		/// <param name="color">The desired color
		/// </param>
		public Edge(Node parent, Node child, double abbrevation, System.Drawing.Color color)
		{
			owner = child;
			neighbor = parent;
			origin = parent.findCenter();
			target = child.findCenter();
			//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			Point2D newOrigin = this.findNewOriginOffset(abbrevation);
			//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			Point2D newTarget = this.findNewTargetOffset(abbrevation);
			origin = newOrigin;
			target = newTarget;
			this.color = color;
		}
		
		/// <summary> Find the new target after shortening. Used by the API. Should never be
		/// called directly.
		/// </summary>
		/// <param name="abbrevation">The value specifying the shortening
		/// </param>
		/// <returns> The target after shortening
		/// </returns>
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		protected internal virtual Point2D findNewTargetOffset(double abbrevation)
		{
			double tXOffset, tYOffset;
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			double x = origin.getX() - target.getX();
			if (x != 0)
			{
				double slope = Slope;
				double xdif = System.Math.Sqrt(abbrevation * abbrevation / (1 + slope * slope));
				double ydif = xdif * slope;
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				if (origin.getX() > target.getX())
				{
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					if (origin.getY() > target.getY())
					{
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						tXOffset = target.getX() + xdif;
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						tYOffset = target.getY() + ydif;
					}
					else
					{
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						tXOffset = target.getX() + xdif;
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						tYOffset = target.getY() + ydif;
					}
				}
				else
				{
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					if (origin.getY() > target.getY())
					{
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						tXOffset = target.getX() - xdif;
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						tYOffset = target.getY() - ydif;
					}
					else
					{
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						tXOffset = target.getX() - xdif;
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						tYOffset = target.getY() - ydif;
					}
				}
			}
			else
			{
				double ydif = abbrevation;
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				if (origin.getY() > target.getY())
				{
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					tYOffset = target.getY() + ydif;
				}
				else
				{
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					tYOffset = target.getY() - ydif;
				}
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				tXOffset = target.getX();
			}
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			return (new System.Drawing.Point((int) tXOffset, (int) tYOffset));
		}
		
		/// <summary> Find the new origin after shortening. Used by the API. Should never be
		/// called directly.
		/// </summary>
		/// <param name="abbrevation">The value specifying the shortening
		/// </param>
		/// <returns> The origin after shortening
		/// </returns>
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		protected internal virtual Point2D findNewOriginOffset(double abbrevation)
		{
			double oXOffset, oYOffset;
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			double x = origin.getX() - target.getX();
			if (x != 0)
			{
				double slope = Slope;
				double xdif = System.Math.Sqrt(abbrevation * abbrevation / (1 + slope * slope));
				double ydif = xdif * slope;
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				if (origin.getX() > target.getX())
				{
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					if (origin.getY() > target.getY())
					{
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						oXOffset = origin.getX() - xdif;
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						oYOffset = origin.getY() - ydif;
					}
					else
					{
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						oXOffset = origin.getX() - xdif;
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						oYOffset = origin.getY() - ydif;
					}
				}
				else
				{
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					if (origin.getY() > target.getY())
					{
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						oXOffset = origin.getX() + xdif;
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						oYOffset = origin.getY() + ydif;
					}
					else
					{
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						oXOffset = origin.getX() + xdif;
						//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						oYOffset = origin.getY() + ydif;
					}
				}
			}
			else
			{
				double ydif = abbrevation;
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				if (origin.getY() > target.getY())
				{
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					oYOffset = origin.getY() - ydif;
				}
				else
				{
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					oYOffset = origin.getY() + ydif;
				}
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				oXOffset = origin.getX();
			}
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			return (new System.Drawing.Point((int) oXOffset, (int) oYOffset));
		}
		
		public virtual void  draw(System.Drawing.Graphics graphics)
		{
			if (!drawSelected_Renamed_Field)
				SupportClass.GraphicsManager.manager.SetColor(graphics, this.Color);
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			graphics.DrawLine(SupportClass.GraphicsManager.manager.GetPen(graphics), (int) origin.getX(), (int) origin.getY(), (int) target.getX(), (int) target.getY());
			SupportClass.GraphicsManager.manager.SetColor(graphics, System.Drawing.Color.Black);
		}
		
		public virtual void  drawSelected(System.Drawing.Graphics graphics)
		{
			SupportClass.GraphicsManager.manager.SetColor(graphics, System.Drawing.Color.Red);
			drawSelected_Renamed_Field = true;
			draw(graphics);
			SupportClass.GraphicsManager.manager.SetColor(graphics, System.Drawing.Color.Black);
		}
		
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual bool isPointWithinShape(Point2D point)
		{
			// Return false if the point is not in the right x range
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			if ((point.getX() > origin.getX() + sens && point.getX() > target.getX() + sens) || (point.getX() < origin.getX() - sens && point.getX() < target.getX() - sens))
				return false;
			// Return false if the point is not in the right y range
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			if ((point.getY() > origin.getY() + sens && point.getY() > target.getY() + sens) || (point.getY() < origin.getY() - sens && point.getY() < target.getY() - sens))
				return false;
			// Otherwise, calculate the distance to the edge
			double slope = Slope;
			double distance;
			// The edge is "vertical": only look at the x-component
			if (slope > 100 || slope < - 100)
			{
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				distance = origin.getX() - point.getX();
				return (distance < sens && distance > - sens);
			}
			// The edge is "horizontal": only look at the x-component
			if (slope > - 0.001 && slope < 0.001)
			{
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				distance = origin.getY() - point.getY();
				return (distance < sens && distance > - sens);
			}
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			distance = (slope * point.getX() + Intercept - point.getY()) / System.Math.Sqrt(slope * slope + 1);
			return (distance < sens && distance > - sens);
		}
		
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual Drawable getSelectedItem(Point2D point)
		{
			if (this.isPointWithinShape(point))
				return this;
			return null;
		}
		
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		public virtual SupportClass.CollectionSupport addSelectedItems(System.Drawing.Rectangle r, SupportClass.CollectionSupport allready)
		{
			//UPGRADE_TODO: Method 'java.awt.geom.RectangularShape.contains' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			if (r.contains(this.origin) && r.contains(this.target))
				allready.Add(this);
			return allready;
		}
		
		/// <summary> Returns a string representation of this edge</summary>
		/// <returns> a string representation of this edge.
		/// </returns>
		public override System.String ToString()
		{
			//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			return "(" + neighbor + ", " + owner + ")";
		}
	}
}