using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> A set of edges belonging to a node.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// <p>Company: </p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class EdgeSet : Drawable
	{
		/// <summary> Returns all edges in the set.</summary>
		/// <returns> A collection of all edges in the set.
		/// </returns>
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		virtual public SupportClass.CollectionSupport Edges
		{
			get
			{
				return this.edges;
			}
			
		}
		//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		private System.Collections.ArrayList edges;
		private Edge selected;
		
		/// <summary> Creates an edge set from a node.</summary>
		/// <param name="node">The node that owns the edges.
		/// </param>
		/// <param name="moral">Specify "true" if the set shall contain the undirected edges,
		/// and "false" if the usual directed edges.
		/// </param>
		public EdgeSet(Node node, bool moral)
		{
			//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			Point2D thisCenter = node.findCenter();
			NodeList parents;
			if (moral)
				parents = node.undirectedEdges;
			else
				parents = node.Parents;
			//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			edges = new System.Collections.ArrayList(parents.Count);
			//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			System.Collections.ArrayList edgeTypes = node.edgeTypes;
			int i = 0;
			System.Collections.IEnumerator ite = parents.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node parent = (Node) ite.Current;
				//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				Point2D parentCenter = parent.findCenter();
				if (moral)
				{
					if (((System.String) edgeTypes[i]).Equals("m"))
					{
						//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
						//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						edges.Add(new Edge(parent, node, node.HomeDomain.getNodeSize().getWidth() / 2, System.Drawing.Color.Red));
					}
					else
					{
						if (((System.String) edgeTypes[i]).Equals("t"))
						{
							//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
							//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
							edges.Add(new Edge(parent, node, node.HomeDomain.getNodeSize().getWidth() / 2, System.Drawing.Color.Blue));
						}
						else
						{
							//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
							//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
							edges.Add(new Edge(parent, node, node.HomeDomain.getNodeSize().getWidth() / 2, System.Drawing.Color.Black));
						}
					}
				}
				else
				{
					//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					edges.Add(new DirectedEdge(parent, node, node.HomeDomain.getNodeSize().getWidth() / 2));
				}
				i++;
			}
		}
		
		/// <summary> Adds an edge to the set</summary>
		/// <param name="edge">The edge to be added
		/// </param>
		public virtual void  addEdge(Edge edge)
		{
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			edges.Add(edge);
		}
		
		/// <summary> Removes an edge from the set.</summary>
		/// <param name="edge">The edge to be removed.
		/// </param>
		public virtual void  removeEdge(Edge edge)
		{
			edges.Remove(edge);
		}
		
		/// <summary> Removes an edge linking to a specific node.</summary>
		/// <param name="parent">The specific node
		/// </param>
		public virtual void  removeEdgeFrom(Node parent)
		{
			bool notFound = true;
			System.Collections.IEnumerator ite = edges.GetEnumerator();
			Edge toParent = (Edge) edges[0];
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext() && notFound)
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Edge edge = (Edge) ite.Current;
				if (edge.Neighbor == parent)
				{
					toParent = edge;
					notFound = false;
				}
			}
			this.removeEdge(toParent);
		}
		
		public virtual void  draw(System.Drawing.Graphics graphics)
		{
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			for (int i = 0, stop = edges.Count; i < stop; i++)
				((Edge) edges[i]).draw(graphics);
		}
		
		public virtual void  drawSelected(System.Drawing.Graphics graphics)
		{
			SupportClass.GraphicsManager.manager.SetColor(graphics, System.Drawing.Color.Red);
			draw(graphics);
			SupportClass.GraphicsManager.manager.SetColor(graphics, System.Drawing.Color.Black);
		}
		
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual Drawable getSelectedItem(Point2D point)
		{
			if (this.isPointWithinShape(point))
				return selected;
			return null;
		}
		
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual bool isPointWithinShape(Point2D point)
		{
			selected = null;
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			for (int i = 0, stop = edges.Count; i < stop; i++)
			{
				Edge edge = (Edge) edges[i];
				if (edge.isPointWithinShape(point))
				{
					selected = edge;
					return true;
				}
			}
			return false;
		}
		
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		public virtual SupportClass.CollectionSupport addSelectedItems(System.Drawing.Rectangle r, SupportClass.CollectionSupport allready)
		{
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			for (int i = 0, stop = this.edges.Count; i < stop; i++)
				allready = ((Edge) edges[i]).addSelectedItems(r, allready);
			return allready;
		}
	}
}