using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> The superclass of all chance nodes.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// <p>Company: </p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class LimidChanceNode:DiscreteNode
	{
		/// <summary> Returns the color of a chance node. This default implementation uses
		/// the static field LimidDomain.chanceNodeColor for representation of
		/// this color. Subclasses may override the method for more sofisticated
		/// coloring.
		/// </summary>
		/// <returns> The color.
		/// </returns>
		/// <seealso cref="LimidDomain#chanceNodeColor">
		/// </seealso>
		override protected internal System.Drawing.Color Color
		{
			get
			{
				return this.HomeDomain.chanceNodeColor;
			}
			
		}
		/// <summary> This general chance node class returns the empty string.</summary>
		/// <returns> The empty string
		/// </returns>
		override protected internal System.String Subtype
		{
			get
			{
				return "";
			}
			
		}
		/// <summary> Returns the string "chance".</summary>
		/// <returns> The string "chance".
		/// </returns>
		override public System.String Type
		{
			get
			{
				return "chance";
			}
			
		}
		/// <summary> Used for creating unique default names</summary>
		internal static int counter = 0;
		
		/// <summary> Creates a chance node within the specified domain.</summary>
		/// <param name="ld">The domain.
		/// @throws MLPDuplicateNodeNameException
		/// </param>
		public LimidChanceNode(LimidDomain ld):base(ld)
		{
			counter++;
		}
		
		/// <summary> The defaultname is the letter "C" followed by a number.</summary>
		protected internal override void  setDefaultName()
		{
			name = "C" + counter;
		}
		
		//UPGRADE_TODO: The equivalent of method 'java.lang.Object.clone' is not an override method. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1143"'
		protected internal System.Object Clone()
		{
			try
			{
				LimidChanceNode newNode = new LimidChanceNode(this.HomeDomain);
				newNode.copyNode(this);
				return newNode;
			}
			catch (MLPException ex)
			{
				return null;
			}
		}
	}
}