using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> The class representing a full Limid (also called a domain or a model).
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class LimidDomain : Drawable
	{
		private void  InitBlock()
		{
			chanceNodeColor = System.Drawing.Color.Yellow;
			decisionNodeColor = System.Drawing.Color.Green;
			utilityNodeColor = System.Drawing.Color.Magenta;
		}
		//UPGRADE_NOTE: Respective javadoc comments were merged.  It should be changed in order to comply with .NET documentation conventions.
		/// <summary> Returns the current chance node color.</summary>
		/// <returns> The color.
		/// </returns>
		/// <summary> Defines the chance node color.</summary>
		/// <param name="color">The desired color.
		/// </param>
		virtual public System.Drawing.Color ChanceNodeColor
		{
			get
			{
				return chanceNodeColor;
			}
			
			set
			{
				chanceNodeColor = value;
			}
			
		}
		//UPGRADE_NOTE: Respective javadoc comments were merged.  It should be changed in order to comply with .NET documentation conventions.
		/// <summary> Returns the current decision node color.</summary>
		/// <returns> The color.
		/// </returns>
		/// <summary> Defines the decision node color.</summary>
		/// <param name="color">The desired color.
		/// </param>
		virtual public System.Drawing.Color DecisionNodeColor
		{
			get
			{
				return decisionNodeColor;
			}
			
			set
			{
				decisionNodeColor = value;
			}
			
		}
		//UPGRADE_NOTE: Respective javadoc comments were merged.  It should be changed in order to comply with .NET documentation conventions.
		/// <summary> Returns the current utility node color.</summary>
		/// <returns> The color.
		/// </returns>
		/// <summary> Defines the utility node color.</summary>
		/// <param name="color">The desired color.
		/// </param>
		virtual public System.Drawing.Color UtilityNodeColor
		{
			get
			{
				return utilityNodeColor;
			}
			
			set
			{
				utilityNodeColor = value;
			}
			
		}
		//UPGRADE_NOTE: Respective javadoc comments were merged.  It should be changed in order to comply with .NET documentation conventions.
		/// <summary> Returns the current name of this model.</summary>
		/// <returns> The name.
		/// </returns>
		/// <summary> Sets the name of this model.</summary>
		/// <param name="name">The desired name.
		/// </param>
		virtual public System.String Name
		{
			get
			{
				return name;
			}
			
			set
			{
				this.name = value;
			}
			
		}
		/// <summary> Returns a list of all nodes.</summary>
		/// <returns> A list of nodes.
		/// </returns>
		virtual public NodeList Nodes
		{
			get
			{
				if (nodes.Count > 0)
					return new NodeList(nodes.Values);
				else
					return new NodeList();
			}
			
		}
		/// <summary> Returns the junction tree. If the model is not compiled, null is returned.</summary>
		/// <returns> The junction tree.
		/// </returns>
		virtual public LimidJunctionTree JunctionTree
		{
			get
			{
				return junctionTree;
			}
			
		}
		/// <summary> Returns true if the model is compiled.</summary>
		/// <returns> true if the model is compiled.
		/// </returns>
		virtual public bool Compiled
		{
			get
			{
				return isCompiled_Renamed_Field;
			}
			
		}
		/// <summary> Returns true if the model is moralized.</summary>
		/// <returns> true if the model is compiled.
		/// </returns>
		virtual public bool Moral
		{
			get
			{
				return isMoral_Renamed_Field;
			}
			
		}
		/// <summary> Returns true if the model is triangulated.</summary>
		/// <returns> true if the model is triangulated.
		/// </returns>
		virtual public bool Triangulated
		{
			get
			{
				return isTriangulated_Renamed_Field;
			}
			
		}
		/// <summary> Called by the compile method. The message paths between all decision nodes
		/// are determined according to the specified ordering. Those paths are used
		/// for partial collect propagation.
		/// </summary>
		/// <param name="ordering">The ordering used for determining the paths.
		/// </param>
		virtual internal NodeList MessagePaths
		{
			set
			{
				LimidClique target = ((DiscreteDecisionChanceNode) value.GetLast()).HomeClique;
				System.Collections.IEnumerator ite = value.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					DiscreteDecisionChanceNode decision = (DiscreteDecisionChanceNode) ite.Current;
					LimidClique present = decision.HomeClique;
					SupportClass.ListCollectionSupport path = present.getPathToClique(null, target);
					decision.PathToPrevious = path;
					target = present;
				}
			}
			
		}
		//UPGRADE_NOTE: Respective javadoc comments were merged.  It should be changed in order to comply with .NET documentation conventions.
		/// <summary> Returns the default ordering used by the Single Policy Updating algorithm.</summary>
		/// <returns> The ordering as a node list.
		/// @throws MLPNetworkNotCompiledException
		/// </returns>
		/// <summary> Defines the default decision node ordering to be used by the Single Policy
		/// Updating algorithm. The specified node list must be a permutation of the
		/// decision nodes of this model.
		/// </summary>
		/// <param name="newOrdering">The desired ordering.
		/// @throws MLPNetworkNotCompiledException If the model is not compiled
		/// @throws MLPInvalidNodeListException If the specified node list is not a
		/// permutation of the decision nodes.
		/// </param>
		virtual public NodeList DefaultOrdering
		{
			get
			{
				if (Compiled)
					return this.defaultOrdering;
				else
					throw new MLPNetworkNotCompiledException();
			}
			
			set
			{
				if (!isCompiled_Renamed_Field)
					throw new MLPNetworkNotCompiledException();
				if (!(decisionNodes.ContainsAll(value) && value.ContainsAll(decisionNodes)))
					throw new MLPInvalidNodeListException("The specified node list is not " + "a permutation of the decision " + "nodes of this model.");
				this.defaultOrdering = value;
				this.MessagePaths = defaultOrdering;
			}
			
		}
		/// <summary> Returns the list of utility nodes</summary>
		/// <returns> A list of utility nodes
		/// @throws MLPNetworkNotCompiledException
		/// </returns>
		virtual public NodeList UtilityNodes
		{
			get
			{
				if (!isCompiled_Renamed_Field)
					throw new MLPNetworkNotCompiledException();
				return this.utilityNodes;
			}
			
		}
		/// <summary> Returns an array of extreme positions of the nodes.</summary>
		/// <returns> An array containg: The minimum x-position, the maximum x-position,
		/// the minimum y-position, the maximum y-position.
		/// </returns>
		virtual public double[] ExtremePositions
		{
			get
			{
				NodeList nodes = this.Nodes;
				System.Collections.IEnumerator ite = nodes.GetEnumerator();
				double minX = 1e300, minY = 1e300, maxX = - 1e300, maxY = - 1e300;
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node node = (Node) ite.Current;
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					double thisX = node.getPosition().getX();
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					double thisY = node.getPosition().getY();
					if (thisX < minX)
						minX = thisX;
					if (thisX > maxX)
						maxX = thisX;
					if (thisY < minY)
						minY = thisY;
					if (thisY > maxY)
						maxY = thisY;
				}
				return new double[]{minX, maxX, minY, maxY};
			}
			
		}
		/// <summary> A triangulation criterion. A concrete criterion must extend this class.</summary>
		/// <author>  Anders Ringgaard Kristensen
		/// </author>
		/// <version>  1.0
		/// </version>
		public abstract class TriangulationCriterion
		{
			/// <summary> A criterion used by the triangulation method.</summary>
			/// <param name="node">The node being considered
			/// </param>
			/// <returns> A value describing the suitability of the node.
			/// </returns>
			protected internal abstract int criterion(DiscreteNode node);
		}
		/// <summary> The default triangulation criterion. Currently the only defined criterion.</summary>
		/// <author>  Anders Ringgaard Kristensen
		/// </author>
		/// <version>  1.0
		/// </version>
		public class MinimumCardinalityCriterion:LimidDomain.TriangulationCriterion
		{
			/// <summary> A criterion used by the triangulation method. This version minimizes total
			/// cardinality of family. Subclasses may overwrite this method in order to have
			/// other critera.
			/// </summary>
			/// <param name="node">The node being considered
			/// </param>
			/// <returns> A value describing the suitability of the node.
			/// </returns>
			protected internal override int criterion(DiscreteNode node)
			{
				System.Collections.IEnumerator ite = node.undirectedEdges.GetEnumerator();
				int cardinality = 1;
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					DiscreteNode neighbor = (DiscreteNode) ite.Current;
					if (neighbor.triangulationNumber < 1)
						cardinality = cardinality * neighbor.getNumberOfStates();
				}
				cardinality = cardinality * node.getNumberOfStates();
				return cardinality;
			}
		}
		
		/// <summary> The minimum cardinality criterion</summary>
		//UPGRADE_NOTE: Final was removed from the declaration of 'MINIMUM_CARDINALITY '. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1003"'
		public static readonly LimidDomain.TriangulationCriterion MINIMUM_CARDINALITY = new LimidDomain.MinimumCardinalityCriterion();
		
		/// <summary> The triangulation criterion used by this model</summary>
		protected internal LimidDomain.TriangulationCriterion criterion_Renamed_Field;
		
		/// <summary> The nodes of the network. The nodes are stored in a HashMap with the unique
		/// name of the node as the key.
		/// </summary>
		//UPGRADE_TODO: Class 'java.util.HashMap' was converted to 'System.Collections.Hashtable' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilHashMap"'
		protected internal System.Collections.Hashtable nodes;
		
		/// <summary> The size of a node. Currently only circles are allowed for nodes, but
		/// later it may be relevant to define both height and width, so a Dimension
		/// is used to represent the size.
		/// </summary>
		protected internal System.Drawing.Size nodeSize;
		
		/// <summary> A list of the utility nodes of this model.</summary>
		protected internal NodeList utilityNodes;
		
		/// <summary> A list of the decision nodes of this model.</summary>
		protected internal NodeList decisionNodes;
		
		/// <summary> A specific chance node defined to be the input node of this Limid.
		/// Currently the input node is not used, but it will be needed for integration
		/// with Markov decision processes.
		/// </summary>
		protected internal LimidChanceNode inputNode;
		
		/// <summary> A specific chance node defined to be the output node of this Limid.
		/// Currently the output node is not used, but it will be needed for integration
		/// with Markov decision processes.
		/// </summary>
		protected internal LimidChanceNode outputNode;
		
		/// <summary> The clique (when compiled) containing the input node.</summary>
		protected internal LimidClique inputClique;
		
		/// <summary> A list of decision nodes defining the default optimization ordering used
		/// by single policy updating.
		/// </summary>
		protected internal NodeList defaultOrdering;
		
		/// <summary> The junction tree of the model (when compiled).</summary>
		protected internal LimidJunctionTree junctionTree = null;
		
		/// <summary> A descriptive name for this model.</summary>
		protected internal System.String name = "Unnamed";
		
		/// <summary> If this flag is set, the draw method draws the moral (or triangulated)
		/// graph of this model. If not set, the directed graph is drawn.
		/// </summary>
		public static bool showMoral = false;
		
		/// <summary> If this flag is set, the draw method draws the junction tree
		/// of this model. If not set, the directed graph is drawn.
		/// </summary>
		public static bool showJunction = false;
		
		/// <summary> This flag is set when the model has been compiled.</summary>
		protected internal bool isCompiled_Renamed_Field = false;
		
		/// <summary> This flag is set when the model has been moralized.</summary>
		protected internal bool isMoral_Renamed_Field = false;
		
		/// <summary> This flag is set when the model has been triangulated.</summary>
		protected internal bool isTriangulated_Renamed_Field = false;
		
		/// <summary> The color used for drawing chance nodes. The default is yellow.</summary>
		//UPGRADE_NOTE: The initialization of  'chanceNodeColor' was moved to method 'InitBlock'. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1005"'
		protected internal System.Drawing.Color chanceNodeColor;
		
		/// <summary> The color used for drawing decision nodes. The default is green.</summary>
		//UPGRADE_NOTE: The initialization of  'decisionNodeColor' was moved to method 'InitBlock'. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1005"'
		protected internal System.Drawing.Color decisionNodeColor;
		
		/// <summary> The color used for drawing utility nodes. The default is magenta.</summary>
		//UPGRADE_NOTE: The initialization of  'utilityNodeColor' was moved to method 'InitBlock'. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1005"'
		protected internal System.Drawing.Color utilityNodeColor;
		
		/// <summary> Creates a new empty Limid.</summary>
		public LimidDomain()
		{
			InitBlock();
			//UPGRADE_TODO: Class 'java.util.HashMap' was converted to 'System.Collections.Hashtable' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilHashMap"'
			nodes = new System.Collections.Hashtable();
			nodeSize = new System.Drawing.Size(40, 40);
			criterion_Renamed_Field = MINIMUM_CARDINALITY;
		}
		
		/// <summary> Creates a new empty Limid with a specified name.</summary>
		/// <param name="name">The desired name.
		/// </param>
		public LimidDomain(System.String name):this()
		{
			Name = name;
		}
		
		/// <summary> Adds a specified node to the Limid.</summary>
		/// <param name="node">The node to be added
		/// @throws MLPDuplicateNodeNameException If the name of the new node conflicts
		/// with the name of an existing node.
		/// </param>
		public virtual void  addNode(Node node)
		{
			if (nodes.ContainsKey(node.getName()))
				throw new MLPDuplicateNodeNameException();
			SupportClass.PutElement(nodes, node.getName(), node);
		}
		
		/// <summary> Returns the node having the specified unique name.</summary>
		/// <param name="name">The name of the node to be found.
		/// </param>
		/// <returns> The node
		/// @throws MLPNodeNotFoundException If no node has the name
		/// </returns>
		public virtual Node getNodeByName(System.String name)
		{
			//UPGRADE_TODO: Method 'java.util.HashMap.get' was converted to 'System.Collections.Hashtable.this' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilHashMapget_javalangObject"'
			Node node = (Node) nodes[name];
			if (node == null)
				throw new MLPNodeNotFoundException(name);
			return node;
		}
		
		/// <summary> Returns the node size of this model.</summary>
		/// <returns> The node size.
		/// </returns>
		public virtual System.Drawing.Size getNodeSize()
		{
			return nodeSize;
		}
		
		/// <summary> Renames a node (identified by its old name).</summary>
		/// <param name="oldName">The old name
		/// </param>
		/// <param name="newName">The new name
		/// @throws MLPDuplicateNodeNameException If the new name conflicts with the
		/// name of an other node.
		/// </param>
		protected internal virtual void  renameNode(System.String oldName, System.String newName)
		{
			try
			{
				Node existing = getNodeByName(newName);
				// If the try-block is still alive, the new name already exists
				throw new MLPDuplicateNodeNameException();
			}
			catch (MLPNodeNotFoundException ex)
			{
				// This is the desired (and normal) situation
				Node node = (Node) SupportClass.HashtableRemove(nodes, oldName);
				node.name = newName;
				SupportClass.PutElement(nodes, node.getName(), node);
			}
		}
		
		/// <summary> Moralize this Limid.</summary>
		public virtual void  moralize()
		{
			// Add undirected edges for all parent/child relations
			System.Collections.IEnumerator ite = this.Nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				node.triangulationNumber = - 1;
				// The create method also deletes any existing undirected edges
				node.createFirstUndirectedEdges();
			}
			// Add moral links
			ite = this.Nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				Node[] parents = new Node[node.Parents.Count];
				parents = (Node[]) node.Parents.ToArray(parents);
				for (int i = 0; i < parents.Length; i++)
					for (int j = 0; j < parents.Length; j++)
						if (i != j && (!parents[i].isConnectedTo(parents[j])))
							parents[i].addUndirectedEdge(parents[j], "m");
			}
			// Remove edges to & from utility nodes
			ite = this.Nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				if (node is TimeUtilityNode)
				{
					TimeUtilityNode utility = (TimeUtilityNode) node;
					utility.removeUndirectedEdges();
					NodeList utParents = utility.Parents;
					System.Collections.IEnumerator jte = utParents.GetEnumerator();
					//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
					while (jte.MoveNext())
					{
						//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
						Node utParent = (Node) jte.Current;
						utParent.removeUndirectedEdge(utility);
					}
				}
			}
			isMoral_Renamed_Field = true;
		}
		
		/// <summary> De-moralize this Limid. The moral edges are removed and the "isMoral" flag
		/// is set to false.
		/// </summary>
		public virtual void  deMoralize()
		{
			// The de-moralization just means that all undirected edges are removed
			System.Collections.IEnumerator ite = this.Nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				node.removeUndirectedEdges();
			}
			isMoral_Renamed_Field = false;
		}
		
		/// <summary> Triangulate this Limid. If it is not moralized, the moralize method is
		/// called initially.
		/// </summary>
		public virtual void  triangulate()
		{
			// Moralize if not done
			if (!isMoral_Renamed_Field)
				this.moralize();
			NodeList unNumbered = new NodeList(this.Nodes);
			// Ignore utility nodes
			SupportClass.ListCollectionSupport utilities = new SupportClass.ListCollectionSupport();
			System.Collections.IEnumerator iterator = unNumbered.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (iterator.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) iterator.Current;
				if (node is TimeUtilityNode)
					utilities.Add(node);
			}
			unNumbered.RemoveAll(utilities);
			NodeList numbered = new NodeList();
			int counter = unNumbered.Count;
			while (counter > 0)
			{
				System.Collections.IEnumerator ite = unNumbered.GetEnumerator();
				double c = 1.0e300;
				DiscreteNode candidate = (DiscreteNode) unNumbered[0];
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					DiscreteNode node = (DiscreteNode) ite.Current;
					if (node.triangulationNumber < 1)
					{
						int crit = criterion(node);
						if (crit < c)
						{
							c = crit;
							candidate = node;
						}
					}
				}
				// We found the best candidate until now
				unNumbered.RemoveElement(candidate);
				numbered.Add(candidate);
				candidate.triangulationNumber = counter;
				
				NodeList clique = new NodeList();
				clique.Add(candidate);
				NodeList neighbors = candidate.undirectedEdges;
				ite = neighbors.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					DiscreteNode node = (DiscreteNode) ite.Current;
					if (node.triangulationNumber < 1)
						clique.Add(node);
				}
				DiscreteNode[] nodeSet = new DiscreteNode[clique.Count];
				nodeSet = (DiscreteNode[]) clique.ToArray(nodeSet);
				for (int i = 0; i < nodeSet.Length; i++)
					for (int j = 0; j < nodeSet.Length; j++)
						if (i != j)
						{
							if (!nodeSet[i].isConnectedTo(nodeSet[j]))
								nodeSet[i].addUndirectedEdge(nodeSet[j], "t");
						}
				counter--;
			}
			isTriangulated_Renamed_Field = true;
			isMoral_Renamed_Field = false;
		}
		
		/// <summary> De-triangulate this model. First all undirected edges are removed. Since
		/// this also removes the moralization, the moralize method is called before
		/// termination.
		/// </summary>
		public virtual void  deTriangulate()
		{
			// We cannot distinguish the undirected edges, so we first remove all
			// and then re-moralize.
			this.deMoralize();
			this.moralize();
			isTriangulated_Renamed_Field = false;
		}
		
		
		/// <summary> Create the cliques of this model. If it is not triangulated it is done.
		/// This method is not used by the API, since no junction tree is created.
		/// </summary>
		/// <returns> The list of cliques in arbitrary order.
		/// </returns>
		public virtual SupportClass.ListCollectionSupport createCliques()
		{
			if (!this.isTriangulated_Renamed_Field)
			{
				if (!this.isMoral_Renamed_Field)
					this.moralize();
				this.triangulate();
			}
			// Establish list of all chance & decision nodes
			NodeList allNodes = new NodeList(this.Nodes);
			// Ignore utility nodes
			SupportClass.ListCollectionSupport utilities = new SupportClass.ListCollectionSupport();
			System.Collections.IEnumerator iterator = allNodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (iterator.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) iterator.Current;
				if (node is TimeUtilityNode)
					utilities.Add(node);
			}
			allNodes.RemoveAll(utilities);
			
			SupportClass.ListCollectionSupport cliqueCandidates = new SupportClass.ListCollectionSupport();
			
			while (allNodes.Count > 0)
			{
				System.Collections.IEnumerator ite = allNodes.GetEnumerator();
				bool notFound = true;
				DiscreteNode simplicial = (DiscreteNode) allNodes[0];
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext() && notFound)
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					DiscreteNode node = (DiscreteNode) ite.Current;
					if (node.isSimplicialWithin(allNodes))
					{
						simplicial = node;
						notFound = false;
					}
				}
				NodeList family = new NodeList();
				NodeList originalFamily = simplicial.undirectedEdges;
				// Prune for already eliminated neighbors
				ite = originalFamily.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					DiscreteNode node = (DiscreteNode) ite.Current;
					if (allNodes.Contains(node))
						family.Add(node);
				}
				family.Add(simplicial);
				allNodes.RemoveElement(simplicial);
				cliqueCandidates.Add(family);
			}
			SupportClass.ListCollectionSupport prunes = new SupportClass.ListCollectionSupport();
			SupportClass.ListCollectionSupport[] candArray = new SupportClass.ListCollectionSupport[cliqueCandidates.Count];
			candArray = (SupportClass.ListCollectionSupport[]) cliqueCandidates.ToArray(candArray);
			for (int i = 0; i < candArray.Length; i++)
				for (int j = 0; j < candArray.Length; j++)
					if (i != j)
					{
						if (candArray[i].ContainsAll(candArray[j]))
						{
							if (candArray[i].Count != candArray[j].Count)
								prunes.Add(candArray[j]);
							else
							{
								//The two candidates are identical - remove only one
								if (i < j)
									prunes.Add(candArray[j]);
							}
						}
					}
			// Remove the subsets
			cliqueCandidates.RemoveAll(prunes);
			return cliqueCandidates;
		}
		
		/// <summary> Compiles this model by defining arbitrary input and output nodes and
		/// afterwards calling the following compile metod.
		/// @throws MLPOperationNotAllowedException
		/// @throws MLPGraphNotConnectedException
		/// </summary>
		public virtual void  compile()
		{
			NodeList nodes = this.Nodes;
			System.Collections.IEnumerator ite = nodes.GetEnumerator();
			LimidChanceNode input = null;
			LimidChanceNode output = null;
			bool notFoundInput = true;
			bool notFoundOutput = true;
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext() && (notFoundInput || notFoundOutput))
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				if (node is LimidChanceNode)
				{
					if (notFoundInput)
					{
						input = (LimidChanceNode) node;
						notFoundInput = false;
					}
					else
					{
						output = (LimidChanceNode) node;
						notFoundOutput = false;
					}
				}
			}
			if (output == null)
				output = input;
			try
			{
				this.compile(input, output);
			}
			catch (MLPNodeNotMemberException ex)
			{
				throw new MLPImpossibleException(ex);
			}
		}
		
		
		/// <summary> Compiles this model defining two specified nodes to be the input and
		/// output nodes, respectively. If the model is not triangulated, it is done.
		/// </summary>
		/// <param name="input">The input node
		/// </param>
		/// <param name="output">The output node
		/// @throws MLPOperationNotAllowedException
		/// @throws MLPGraphNotConnectedException If the graph is not connected.
		/// @throws MLPNodeNotMemberException If tany of the nodes are not members
		/// </param>
		public virtual void  compile(LimidChanceNode input, LimidChanceNode output)
		{
			// Make sure that the graph is connected
			if (!isConnected(input))
				throw new MLPGraphNotConnectedException(input);
			// Perform the heavy graph theoretical operations (if not already done)
			if (!this.isTriangulated_Renamed_Field)
			{
				if (!this.isMoral_Renamed_Field)
					this.moralize();
				this.triangulate();
			}
			junctionTree = new LimidJunctionTree(this);
			
			this.setInputOutputNodes(input, output);
			// Establist lists of decision and utility nodes
			decisionNodes = new NodeList();
			utilityNodes = new NodeList();
			NodeList nodes = this.Nodes;
			System.Collections.IEnumerator ite = nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				if (node is DiscreteDecisionChanceNode)
					decisionNodes.Add(node);
				if (node is TimeUtilityNode)
					utilityNodes.Add(node);
			}
			// Create default optimization ordering of decisions
			defaultOrdering = new NodeList();
			//UPGRADE_TODO: Method 'java.util.LinkedList.listIterator' was converted to 'System.Collections.ArrayList.GetEnumerator' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073"'
			System.Collections.IEnumerator iterator = decisionNodes.GetEnumerator(decisionNodes.Count, decisionNodes.Count - decisionNodes.Count);
			//UPGRADE_ISSUE: Method 'java.util.ListIterator.hasPrevious' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1000_javautilListIteratorhasPrevious"'
			while (iterator.hasPrevious())
			{
				//UPGRADE_ISSUE: Method 'java.util.ListIterator.previous' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1000_javautilListIteratorprevious"'
				defaultOrdering.Add(iterator.previous());
			}
			
			// Find the clique containing the family of the input node
			SupportClass.ListCollectionSupport lcl = junctionTree.Cliques;
			ite = lcl.GetEnumerator();
			NodeList family = inputNode.Table.Nodes;
			bool notFound = true;
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext() && notFound)
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				LimidClique clique = (LimidClique) ite.Current;
				NodeList cliqueMembers = clique.Members;
				if (cliqueMembers.ContainsAll(family))
				{
					notFound = false;
					this.inputClique = clique;
				}
			}
			
			ite = decisionNodes.GetEnumerator();
			
			if (defaultOrdering.Count > 0)
				this.MessagePaths = defaultOrdering;
			isCompiled_Renamed_Field = true;
		}
		
		/// <summary> De-compile this model. All structures from the compilation are removed.</summary>
		public virtual void  deCompile()
		{
			utilityNodes = null;
			decisionNodes = null;
			inputNode = null;
			outputNode = null;
			inputClique = null;
			defaultOrdering = null;
			junctionTree = null;
			isCompiled_Renamed_Field = false;
		}
		
		/// <summary> A criterion used by the triangulation method. In this implementation
		/// the corresponding method of the inner class representing the triangulation
		/// criterion is called.
		/// </summary>
		/// <param name="node">The node being considered
		/// </param>
		/// <returns> A value describing the suitability of the node.
		/// </returns>
		protected internal virtual int criterion(DiscreteNode node)
		{
			return criterion.criterion(node);
		}
		
		/// <summary> Defines the triangulation criterion to be used by this model</summary>
		/// <param name="tc">The desired criterion.
		/// </param>
		public virtual void  setTriangulationCriterion(LimidDomain.TriangulationCriterion tc)
		{
			criterion_Renamed_Field = tc;
		}
		
		/// <summary> Performs one iteration of the Single Policy Updating algorithm. Each decision
		/// is visited once.
		/// </summary>
		/// <param name="ordering">The decision node ordering to be used
		/// </param>
		/// <param name="first">A flag indicating that the method should use full collect
		/// propagation (instead of partial).
		/// </param>
		/// <returns> A record for the optimization log
		/// @throws MLPOperationNotAllowedException
		/// </returns>
		public virtual SPURecord singlePolicyUpdatingIteration(NodeList ordering, bool first)
		{
			if (!isCompiled_Renamed_Field)
				throw new MLPNetworkNotCompiledException();
			SPURecord result = new SPURecord(ordering.Count);
			System.Collections.IEnumerator ite = ordering.GetEnumerator();
			int i = 0;
			double value_Renamed = 0.0;
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				DiscreteDecisionChanceNode decNode = (DiscreteDecisionChanceNode) ite.Current;
				value_Renamed = decNode.singlePolicyUpdating(first);
				result.setValue(i, value_Renamed);
				first = false;
				i++;
			}
			result.setUtility(value_Renamed);
			return result;
		}
		
		/// <summary> Performs one iteration of the Single Policy Updating algorithm. Each decision
		/// is visited once. The default decision node ordering is used.
		/// </summary>
		/// <param name="first">A flag indicating that the method should use full collect
		/// propagation (instead of partial).
		/// </param>
		/// <returns> A record for the optimization log
		/// @throws MLPOperationNotAllowedException
		/// </returns>
		/// <seealso cref="LimidDomain#defaultOrdering">
		/// </seealso>
		public virtual SPURecord singlePolicyUpdatingIteration(bool first)
		{
			if (!isCompiled_Renamed_Field)
				throw new MLPNetworkNotCompiledException();
			NodeList reverse = new NodeList();
			//UPGRADE_TODO: Method 'java.util.LinkedList.listIterator' was converted to 'System.Collections.ArrayList.GetEnumerator' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073"'
			System.Collections.IEnumerator ite = decisionNodes.GetEnumerator(decisionNodes.Count, decisionNodes.Count - decisionNodes.Count);
			//UPGRADE_ISSUE: Method 'java.util.ListIterator.hasPrevious' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1000_javautilListIteratorhasPrevious"'
			while (ite.hasPrevious())
			{
				//UPGRADE_ISSUE: Method 'java.util.ListIterator.previous' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1000_javautilListIteratorprevious"'
				reverse.Add(ite.previous());
			}
			SPURecord value_Renamed = singlePolicyUpdatingIteration(reverse, first);
			return value_Renamed;
		}
		
		/// <summary> Performs Single Policy Updating until convergence.</summary>
		/// <param name="ordering">The decision node ordering to be used
		/// </param>
		/// <returns> A result object
		/// @throws MLPOperationNotAllowedException
		/// </returns>
		public virtual SPUResult singlePolicyUpdating(NodeList ordering)
		{
			if (!isCompiled_Renamed_Field)
				throw new MLPNetworkNotCompiledException();
			SPUResult result = new SPUResult(new NodeList(ordering));
			// If the ordering is NOT the default, we must first find the new
			// message paths
			if (ordering != defaultOrdering)
				this.MessagePaths = ordering;
			SPURecord utility2 = this.singlePolicyUpdatingIteration(ordering, true);
			result.addRecord(utility2);
			double utility1 = utility2.getUtility() - 1.0;
			while (utility2.getUtility() > utility1)
			{
				utility1 = utility2.getUtility();
				utility2 = this.singlePolicyUpdatingIteration(ordering, false);
				result.addRecord(utility2);
			}
			// Reset message paths
			if (ordering != defaultOrdering)
				this.MessagePaths = defaultOrdering;
			return result;
		}
		
		/// <summary> Performs Single Policy Updating until convergence using the deafault
		/// decision node ordering.
		/// </summary>
		/// <returns> A result object
		/// @throws MLPOperationNotAllowedException
		/// </returns>
		/// <seealso cref="LimidDomain#defaultOrdering">
		/// </seealso>
		public virtual SPUResult singlePolicyUpdating()
		{
			if (!isCompiled_Renamed_Field)
				throw new MLPNetworkNotCompiledException();
			return this.singlePolicyUpdating(defaultOrdering);
		}
		
		public virtual void  draw(System.Drawing.Graphics graphics)
		{
			if (LimidDomain.showJunction)
			{
				junctionTree.draw(graphics);
			}
			else
			{
				NodeList nodes = this.Nodes;
				System.Collections.IEnumerator ite = nodes.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					((Drawable) ite.Current).draw(graphics);
				}
			}
		}
		
		public virtual void  drawSelected(System.Drawing.Graphics graphics)
		{
			draw(graphics);
		}
		
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual Drawable getSelectedItem(Point2D point)
		{
			NodeList nodes = this.Nodes;
			System.Collections.IEnumerator ite = nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Drawable shape = (Drawable) ite.Current;
				Drawable selected = shape.getSelectedItem(point);
				if (selected != null)
					return selected;
			}
			return null;
		}
		
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		public virtual SupportClass.CollectionSupport addSelectedItems(System.Drawing.Rectangle r, SupportClass.CollectionSupport allready)
		{
			NodeList nodes = this.Nodes;
			System.Collections.IEnumerator ite = nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Drawable shape = (Drawable) ite.Current;
				allready = shape.addSelectedItems(r, allready);
			}
			return allready;
		}
		
		/// <summary> Defines the input and output nodes of this domain. Both nodes must already
		/// exist in the domain.
		/// </summary>
		/// <param name="input">The input node
		/// </param>
		/// <param name="output">The output node
		/// @throws MLPNodeNotMemberException If any of the nodes does not belong to
		/// the domain.
		/// </param>
		public virtual void  setInputOutputNodes(LimidChanceNode input, LimidChanceNode output)
		{
			if (input.HomeDomain != this)
				throw new MLPNodeNotMemberException("The input node specified is not " + "member of this domain.");
			if (output.HomeDomain != this)
				throw new MLPNodeNotMemberException("The output node specified is not " + "member of this domain.");
			this.inputNode = input;
			this.outputNode = output;
		}
		
		/// <summary> Checs whether this graph is connected. The test is performed by checking
		/// that all nodes are in some way connected to the start node specified.
		/// </summary>
		/// <param name="startNode">The start node.
		/// </param>
		/// <returns> true if the graph ids connected.
		/// </returns>
		public virtual bool isConnected(Node startNode)
		{
			NodeList connectedSet = new NodeList();
			NodeList nodes = this.Nodes;
			System.Collections.IEnumerator ite = nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				node.triangulationNumber = - 1;
			}
			startNode.reachNode(connectedSet);
			ite = nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				node.triangulationNumber = - 1;
			}
			return (connectedSet.Count == nodes.Count);
		}
		
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual bool isPointWithinShape(Point2D point)
		{
			return false;
		}
		
		/// <summary> Removes the specified node from the Limid.</summary>
		/// <param name="node">The node to be removed.
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public virtual void  removeNode(Node node)
		{
			// We need to create a new list of parents, because the node.removeParent()
			// method changes the node's own list of parents
			NodeList parents = new NodeList(node.Parents);
			System.Collections.IEnumerator ite = parents.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node parent = (Node) ite.Current;
				node.removeParent(parent);
			}
			// We need to create a new list of children, because the child.removeParent()
			// method changes the node's own list of children
			NodeList children = new NodeList(node.Children);
			ite = children.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node child = (Node) ite.Current;
				child.removeParent(node);
			}
			SupportClass.HashtableRemove(nodes, node.getName());
		}
		
		/// <summary> Writes this Limid to a Hugin NET file.</summary>
		/// <param name="filename">The full name of the file.
		/// @throws IOException
		/// </param>
		public virtual void  toNetFile(System.String filename)
		{
			System.IO.FileStream fos = new System.IO.FileStream(filename, System.IO.FileMode.Create);
			System.IO.BufferedStream bos = new System.IO.BufferedStream(fos);
			System.IO.StreamWriter owr = new System.IO.StreamWriter(fos);
			this.toNet(owr);
			owr.Flush();
			owr.Close();
		}
		
		/// <summary> Writes this Limid to the specified output stream. The format is in
		/// accordance with a Hugin NET file.
		/// </summary>
		/// <param name="stream">An output stream writer.
		/// @throws IOException
		/// </param>
		public virtual void  toNet(System.IO.StreamWriter stream)
		{
			NodeList nodes = this.Nodes;
			stream.Write("net\n");
			stream.Write("{\n");
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.Dimension.getHeight' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			stream.Write("    node_size = (" + (int) this.getNodeSize().getWidth() + " " + (int) this.getNodeSize().getHeight() + ");\n");
			stream.Write("}\n\n");
			System.Collections.IEnumerator ite = nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				node.toNet(stream);
			}
			ite = nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				node.potentialToNet(stream);
			}
		}
		
		/// <summary> Writes this Limid to an output stream. The format is in accordance with
		/// the XML format for Bayesian Networks.
		/// </summary>
		/// <param name="blanks">A string of blanks used for indentication. Usually the
		/// empty string.
		/// </param>
		/// <param name="stream">An output stream writer.
		/// @throws IOException
		/// </param>
		public virtual void  toStream(System.String blanks, System.IO.StreamWriter stream)
		{
			stream.Write(blanks + "<?xml version=\"1.0\"?>\n");
			stream.Write(blanks + "<!DOCTYPE ANALYSISNOTEBOOK SYSTEM \"xbn.dtd\">\n");
			stream.Write(blanks + "<ANALYSISNOTEBOOK NAME=\"Notebook." + Name + "\" ROOT=\"" + Name + "\">\n");
			blanks = System.String.Concat(blanks, "  ");
			stream.Write(blanks + "<BNMODEL NAME=\"" + Name + "\">\n");
			blanks = System.String.Concat(blanks, "  ");
			stream.Write(blanks + "<STATICPROPERTIES>\n");
			blanks = System.String.Concat(blanks, "  ");
			stream.Write(blanks + "<FORMAT VALUE = \"MSR DTAS XML\"/>\n");
			stream.Write(blanks + "<VERSION VALUE = \"0.2\"/>\n");
			stream.Write(blanks + "<CREATOR VALUE = \"Esthauge Limid Editor\"/>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</STATICPROPERTIES>\n");
			stream.Write(blanks + "<DYNAMICPROPERTIES>\n");
			blanks = System.String.Concat(blanks, "  ");
			stream.Write(blanks + "<PROPERTYTYPE NAME = \"nodetype\" TYPE = \"enumeration\"\n");
			blanks = System.String.Concat(blanks, "  ");
			blanks = System.String.Concat(blanks, "  ");
			stream.Write(blanks + "ENUMSET =\"chance decision utility\">\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "<COMMENT>The 3 basic node types of a LIMID</COMMENT>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</PROPERTYTYPE>\n");
			stream.Write(blanks + "<PROPERTYTYPE NAME = \"subtype\" TYPE = \"enumeration\"\n");
			blanks = System.String.Concat(blanks, "  ");
			blanks = System.String.Concat(blanks, "  ");
			stream.Write(blanks + "ENUMSET =\"boolean labelled numbered\">\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "<COMMENT>The 3 subtypes of discrete nodes</COMMENT>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</PROPERTYTYPE>\n");
			stream.Write(blanks + "<PROPERTYTYPE NAME = \"statevalue\" TYPE = \"real\">\n");
			blanks = System.String.Concat(blanks, "  ");
			stream.Write(blanks + "<COMMENT>The state values of numbered nodes</COMMENT>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</PROPERTYTYPE>\n");
			stream.Write(blanks + "<PROPERTYTYPE NAME = \"nodesize\" TYPE = \"real\">\n");
			blanks = System.String.Concat(blanks, "  ");
			stream.Write(blanks + "<COMMENT>The diameter of a node</COMMENT>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</PROPERTYTYPE>\n");
			stream.Write(blanks + "<PROPERTYTYPE NAME = \"chancenodecolor\" TYPE = \"real\">\n");
			blanks = System.String.Concat(blanks, "  ");
			stream.Write(blanks + "<COMMENT>The color of a chance node</COMMENT>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</PROPERTYTYPE>\n");
			stream.Write(blanks + "<PROPERTYTYPE NAME = \"decisionnodecolor\" TYPE = \"real\">\n");
			blanks = System.String.Concat(blanks, "  ");
			stream.Write(blanks + "<COMMENT>The color of a decision node</COMMENT>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</PROPERTYTYPE>\n");
			stream.Write(blanks + "<PROPERTYTYPE NAME = \"utilitynodecolor\" TYPE = \"real\">\n");
			blanks = System.String.Concat(blanks, "  ");
			stream.Write(blanks + "<COMMENT>The color of a utility node</COMMENT>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</PROPERTYTYPE>\n");
			stream.Write(blanks + "<PROPERTY NAME = \"nodesize\">\n");
			blanks = System.String.Concat(blanks, "  ");
			stream.Write(blanks + "<PROPVALUE>" + getNodeSize().Width + "</PROPVALUE>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</PROPERTY>\n");
			stream.Write(blanks + "<PROPERTY NAME = \"chancenodecolor\">\n");
			blanks = System.String.Concat(blanks, "  ");
			//UPGRADE_TODO: The equivalent in .NET for method 'java.awt.Color.getRGB' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			stream.Write(blanks + "<PROPVALUE>" + chanceNodeColor.ToArgb() + "</PROPVALUE>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</PROPERTY>\n");
			stream.Write(blanks + "<PROPERTY NAME = \"decisionnodecolor\">\n");
			blanks = System.String.Concat(blanks, "  ");
			//UPGRADE_TODO: The equivalent in .NET for method 'java.awt.Color.getRGB' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			stream.Write(blanks + "<PROPVALUE>" + decisionNodeColor.ToArgb() + "</PROPVALUE>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</PROPERTY>\n");
			stream.Write(blanks + "<PROPERTY NAME = \"utilitynodecolor\">\n");
			blanks = System.String.Concat(blanks, "  ");
			//UPGRADE_TODO: The equivalent in .NET for method 'java.awt.Color.getRGB' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			stream.Write(blanks + "<PROPVALUE>" + utilityNodeColor.ToArgb() + "</PROPVALUE>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</PROPERTY>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</DYNAMICPROPERTIES>\n");
			// Write variables to file
			stream.Write(blanks + "<VARIABLES>\n");
			blanks = System.String.Concat(blanks, "  ");
			System.Collections.IEnumerator ite = Nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				node.toStream(blanks, stream);
			}
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</VARIABLES>\n");
			// Write edges to file
			stream.Write(blanks + "<STRUCTURE>\n");
			blanks = System.String.Concat(blanks, "  ");
			ite = Nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				NodeList parents = node.Parents;
				System.Collections.IEnumerator jte = parents.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (jte.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node parent = (Node) jte.Current;
					stream.Write(blanks + "<ARC PARENT=\"" + parent.getName() + "\" CHILD=\"" + node.getName() + "\"/>\n");
				}
				if (parents.Count == 0 && node.Children.Count == 0)
					stream.Write(blanks + "<MEMBER NAME=\"" + node.getName() + "\"/>\n");
			}
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</STRUCTURE>\n");
			// Write edges to file
			stream.Write(blanks + "<DISTRIBUTIONS>\n");
			blanks = System.String.Concat(blanks, "  ");
			ite = Nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				stream.Write(blanks + "<DIST TYPE=\"discrete\">\n");
				blanks = System.String.Concat(blanks, "  ");
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				NodeList parents = node.Parents;
				if (parents.Count > 0)
				{
					stream.Write(blanks + "<CONDSET>\n");
					blanks = System.String.Concat(blanks, "  ");
					System.Collections.IEnumerator jte = parents.GetEnumerator();
					//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
					while (jte.MoveNext())
					{
						//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
						Node parent = (Node) jte.Current;
						stream.Write(blanks + "<CONDELEM NAME=\"" + parent.getName() + "\"/>\n");
					}
					blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
					stream.Write(blanks + "</CONDSET>\n");
					stream.Write(blanks + "<PRIVATE NAME=\"" + node.getName() + "\"/>\n");
					stream.Write(blanks + "<DPIS>\n");
					blanks = System.String.Concat(blanks, "  ");
					Potential table = node.Table;
					table.initialize();
					while (table.hasNextState())
					{
						int[] state = table.nextState();
						if (node is DiscreteNode)
						{
							if (state[state.Length - 1] == 0)
							{
								stream.Write(blanks + "<DPI INDEXES=\"");
								for (int i = 0; i < state.Length - 1; i++)
									stream.Write(" " + state[i]);
								stream.Write(" \">");
							}
						}
						else
						{
							stream.Write(blanks + "<DPI INDEXES=\"");
							for (int i = 0; i < state.Length; i++)
								stream.Write(" " + state[i]);
							stream.Write(" \">");
						}
						if (node is TimeUtilityNode)
						{
							TimeUtilityNode tun = (TimeUtilityNode) node;
							//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
							System.Collections.ArrayList tables = tun.AllPotentials;
							//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
							for (int i = 0; i < tables.Count; i++)
								stream.Write(" " + ((Potential) tables[i]).getDataItem(state));
						}
						else
							stream.Write(" " + table.getDataItem(state));
						int count = 1;
						if (node is DiscreteNode)
						{
							DiscreteNode dis = (DiscreteNode) node;
							count = dis.getNumberOfStates();
						}
						if (state[state.Length - 1] == count - 1 && node is DiscreteNode)
							stream.Write("</DPI>\n");
						if (node is TimeUtilityNode)
							stream.Write("</DPI>\n");
					}
					blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
					stream.Write(blanks + "</DPIS>\n");
				}
				else
				{
					stream.Write(blanks + "<PRIVATE NAME=\"" + node.getName() + "\"/>\n");
					stream.Write(blanks + "<DPIS>\n");
					blanks = System.String.Concat(blanks, "  ");
					Potential table = node.Table;
					table.initialize();
					while (table.hasNextState())
					{
						int[] state = table.nextState();
						if (node is DiscreteNode)
						{
							if (state[state.Length - 1] == 0)
							{
								stream.Write(blanks + "<DPI>");
							}
						}
						else
						{
							stream.Write(blanks + "<DPI>");
						}
						stream.Write(" " + table.getDataItem(state));
						int count = 1;
						if (node is DiscreteNode)
						{
							DiscreteNode dis = (DiscreteNode) node;
							count = dis.getNumberOfStates();
						}
						if (state[state.Length - 1] == count - 1 && node is DiscreteNode)
							stream.Write("</DPI>\n");
						if (node is TimeUtilityNode)
							stream.Write("</DPI>\n");
					}
					blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
					stream.Write(blanks + "</DPIS>\n");
				}
				blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
				stream.Write(blanks + "</DIST>\n");
			}
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</DISTRIBUTIONS>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			
			stream.Write(blanks + "</BNMODEL>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</ANALYSISNOTEBOOK>\n");
		}
		
		/// <summary> Re-calculate the edges for graphical display. Used by the GUI.</summary>
		public virtual void  reDoEdges()
		{
			NodeList nodes = Nodes;
			System.Collections.IEnumerator ite = nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				node.reDoEdges();
			}
		}
		
		/// <summary> Define the diameter of the nodes.</summary>
		/// <param name="size">The diameter
		/// </param>
		public virtual void  setNodeSize(int size)
		{
			this.nodeSize = new System.Drawing.Size(size, size);
			this.reDoEdges();
		}
		
		/// <summary> Used by the GUI to relocate nodes in order to fit a window.</summary>
		/// <param name="xOffset">The distance to move all nodes to the left
		/// </param>
		/// <param name="xScale">The scaling factor for the x-direction.
		/// </param>
		/// <param name="yOffset">The distance to move all nodes upwards.
		/// </param>
		/// <param name="yScale">The scaling factor for the y-direction.
		/// </param>
		/// <param name="moveFirst">Specify true if the position is first moved and then scaled.
		/// </param>
		public virtual void  reLocateToFitDimension(double xOffset, double xScale, double yOffset, double yScale, bool moveFirst)
		{
			NodeList nodes = Nodes;
			System.Collections.IEnumerator ite = nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				double thisX = node.getPosition().getX();
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				double thisY = node.getPosition().getY();
				double newX;
				double newY;
				if (moveFirst)
				{
					newX = (thisX - xOffset) * xScale;
					newY = (thisY - yOffset) * yScale;
				}
				else
				{
					newX = thisX * xScale - xOffset;
					newY = thisY * yScale - yOffset;
				}
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				node.setPosition(new System.Drawing.Point((int) newX, (int) newY));
			}
		}
	}
}