using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> A runtime exception that indicates an unexpected exception. It is thrown if
	/// a checked exception is thrown at a place where it should not be possible.
	/// Usually the condition indicates a BUG in the source code. Please contact
	/// the developer.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	
	public class MLPImpossibleException:System.SystemException
	{
		/// <summary> The exception causing the unexpected condition</summary>
		internal System.Exception originalException;
		
		/// <summary> The exception is always thrown as the result of a checked exception being
		/// caught at a place where it should not happen. The original exception is kept
		/// in a field.
		/// </summary>
		/// <param name="">original
		/// </param>
		public MLPImpossibleException(System.Exception original):base("An unexpected exception of class " + original.GetType().FullName + " was thrown. Please inform the developer about the deatails.")
		{
			originalException = original;
		}
	}
}