using System;
using dk.esthauge.limid.parser;
using dk.kvl.ihh.prodstyr.mlhmp;
namespace dk.esthauge.limid.api
{
	
	/// <summary> A class having methods for reading Limids from XML-files.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class ModelReader
	{
		/// <summary> The parser used for reading</summary>
		internal LimidParser parser;
		
		
		/// <summary> Creates a reader object from a filename.</summary>
		/// <param name="fileName">The full name of the file.
		/// @throws IOException
		/// @throws ParseException
		/// </param>
		public ModelReader(System.String fileName)
		{
			System.IO.StreamReader fr = new System.IO.StreamReader(fileName);
			//UPGRADE_TODO: Expected value of parameters of constructor 'java.io.BufferedReader.BufferedReader' are different in the equivalent in .NET. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1092"'
			System.IO.StreamReader br = new System.IO.StreamReader(fr.BaseStream, System.Text.Encoding.UTF7);
			parser = new LimidParser(fr);
		}
		
		/// <summary> Creates a reader from a File object.</summary>
		/// <param name="file">The File object.
		/// @throws IOException
		/// @throws ParseException
		/// </param>
		public ModelReader(System.IO.FileInfo file)
		{
			System.IO.StreamReader fr = new System.IO.StreamReader(file.FullName);
			//UPGRADE_TODO: Expected value of parameters of constructor 'java.io.BufferedReader.BufferedReader' are different in the equivalent in .NET. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1092"'
			System.IO.StreamReader br = new System.IO.StreamReader(fr.BaseStream, System.Text.Encoding.UTF7);
			parser = new LimidParser(fr);
		}
		
		/// <summary> Creates a reader from any Reader object</summary>
		/// <param name="">stream
		/// @throws ParseException
		/// </param>
		public ModelReader(System.IO.StreamReader stream)
		{
			parser = new LimidParser(stream);
		}
		
		/// <summary> Reads and returns the Limid.</summary>
		/// <returns> The Limid being read.
		/// @throws ParseException
		/// @throws MLPOperationNotAllowedException
		/// @throws MLPDuplicateNodeNameException
		/// @throws MLPNodeNotFoundException
		/// </returns>
		public virtual LimidDomain readLimid()
		{
			LimidDomain model = null;
			bool nativeLimid = false;
			SupportClass.ListCollectionSupport statics = new SupportClass.ListCollectionSupport();
			SupportClass.ListCollectionSupport dynamics = new SupportClass.ListCollectionSupport();
			SupportClass.ListCollectionSupport variables = new SupportClass.ListCollectionSupport();
			SupportClass.ListCollectionSupport structures = new SupportClass.ListCollectionSupport();
			SupportClass.ListCollectionSupport distributions = new SupportClass.ListCollectionSupport();
			SimpleNode root = (SimpleNode) parser.Input();
			//    root.dump("");
			SimpleNode analysisNoteBook = (SimpleNode) root.jjtGetChild(0);
			int i = 0;
			SimpleNode bnn = null;
			bool notFound = true;
			while (i < analysisNoteBook.jjtGetNumChildren() && notFound)
			{
				SimpleNode sn = (SimpleNode) analysisNoteBook.jjtGetChild(i);
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				if (sn.ToString().CompareTo("BayesianNetwork") == 0)
				{
					notFound = false;
					bnn = sn;
				}
				i++;
			}
			for (int j = 0; j < bnn.jjtGetNumChildren(); j++)
			{
				SimpleNode sn = (SimpleNode) bnn.jjtGetChild(j);
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				if (sn.ToString().StartsWith("attribute"))
				{
					// If it is the name, read it
					NodeAttribute na = (NodeAttribute) sn;
					if (System.String.Compare(na.Name, "NAME", false) == 0)
						model = new LimidDomain(na.StringValue);
					else
						model = new LimidDomain();
				}
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				if (sn.ToString().CompareTo("staticProperties") == 0)
					statics.Add(sn);
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				if (sn.ToString().CompareTo("dynamicProperties") == 0)
					dynamics.Add(sn);
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				if (sn.ToString().CompareTo("variables") == 0)
					variables.Add(sn);
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				if (sn.ToString().CompareTo("structure") == 0)
					structures.Add(sn);
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				if (sn.ToString().CompareTo("distributions") == 0)
					distributions.Add(sn);
			}
			SupportClass.ListCollectionSupport attributes = new SupportClass.ListCollectionSupport();
			System.Collections.IEnumerator ite = statics.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				SimpleNode spn = (SimpleNode) ite.Current;
				for (int j = 0; j < spn.jjtGetNumChildren(); j++)
					attributes.Add(spn.jjtGetChild(j));
			}
			ite = attributes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				SimpleNode n = (SimpleNode) ite.Current;
				NodeAttribute na = (NodeAttribute) n.jjtGetChild(0);
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				if (System.String.Compare(n.ToString(), "format", false) == 0 && System.String.Compare(na.StringValue.Trim(), "MSR DTAS XML", false) != 0)
				{
					throw new ParseException("Unknown format specification: " + na.StringValue);
				}
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				if (System.String.Compare(n.ToString(), "version", false) == 0 && System.String.Compare(na.StringValue.Trim(), "0.2", false) != 0)
				{
					throw new ParseException("Unknown version specification: " + na.StringValue);
				}
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				if (System.String.Compare(n.ToString(), "creator", false) == 0 && System.String.Compare(na.StringValue.Trim(), "Esthauge Limid Editor", false) == 0)
				{
					nativeLimid = true;
				}
			}
			
			// The dynamic properties section(s). We seach for native properties.
			// Non-native properties are ignored;
			ite = dynamics.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				SimpleNode n = (SimpleNode) ite.Current;
				// This is a collection of dynamic properties
				for (int j = 0; j < n.jjtGetNumChildren(); j++)
				{
					SimpleNode pro = (SimpleNode) n.jjtGetChild(j);
					// This is a single element - check whether it is a known property
					//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					if (pro.ToString().ToUpper().Equals("property".ToUpper()))
					{
						// It is a property - search for native
						NodeAttribute name = (NodeAttribute) pro.jjtGetChild(0);
						NodeAttribute value_Renamed = (NodeAttribute) pro.jjtGetChild(1);
						// Is it the node size?
						if (name.StringValue.ToUpper().Equals("nodesize".ToUpper()))
						{
							// Try to interpret the value as an number - otherwise ignore
							try
							{
								//UPGRADE_TODO: Format of parameters of constructor 'java.lang.Double.Double' are different in the equivalent in .NET. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1092"'
								int size = (int) System.Double.Parse(value_Renamed.StringValue);
								model.setNodeSize(size);
							}
							catch (System.FormatException ex)
							{
								// Don't bother - we use the standard size
							}
						}
						// Is it the chance node color?
						if (name.StringValue.ToUpper().Equals("chancenodecolor".ToUpper()))
						{
							// Try to interpret the value as a color - otherwise ignore
							try
							{
								int rbg = System.Int32.Parse(value_Renamed.StringValue);
								model.ChanceNodeColor = new Color(rbg, true);
							}
							catch (System.FormatException ex)
							{
								// Don't bother - we use the standard color
							}
						}
						// Is it the decision node color?
						if (name.StringValue.ToUpper().Equals("decisionnodecolor".ToUpper()))
						{
							// Try to interpret the value as a color - otherwise ignore
							try
							{
								int rbg = System.Int32.Parse(value_Renamed.StringValue);
								model.DecisionNodeColor = new Color(rbg, true);
							}
							catch (System.FormatException ex)
							{
								// Don't bother - we use the standard color
							}
						}
						// Is it the utility node color?
						if (name.StringValue.ToUpper().Equals("utilitynodecolor".ToUpper()))
						{
							// Try to interpret the value as a color - otherwise ignore
							try
							{
								int rbg = System.Int32.Parse(value_Renamed.StringValue);
								model.UtilityNodeColor = new Color(rbg, true);
							}
							catch (System.FormatException ex)
							{
								// Don't bother - we use the standard color
							}
						}
					}
				}
			}
			
			// The variables section(s)
			ite = variables.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				SimpleNode n = (SimpleNode) ite.Current;
				// This is a collection of variables
				for (int j = 0; j < n.jjtGetNumChildren(); j++)
				{
					SimpleNode var = (SimpleNode) n.jjtGetChild(j);
					// This is a single variable. We only interpret what makes sense in our context
					System.String nodeName = "";
					System.String nodeLabel = "";
					System.String nodeType = "chance";
					System.String subType = "labelled";
					//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
					System.Collections.ArrayList states = new System.Collections.ArrayList();
					//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
					System.Collections.ArrayList stateValues = new System.Collections.ArrayList();
					int xPos = 0;
					int yPos = 0;
					for (int k = 0; k < var.jjtGetNumChildren(); k++)
					{
						SimpleNode inf = (SimpleNode) var.jjtGetChild(k);
						// The first child is always the attributes
						if (k == 0)
						{
							// Read the variable attributes
							for (int l = 0; l < inf.jjtGetNumChildren(); l++)
							{
								NodeAttribute na = (NodeAttribute) inf.jjtGetChild(l);
								if (System.String.Compare(na.Name, "NAME", false) == 0)
									nodeName = na.StringValue;
								if (System.String.Compare(na.Name, "XPOS", false) == 0)
									xPos = na.Value / 100;
								if (System.String.Compare(na.Name, "YPOS", false) == 0)
									yPos = na.Value / 100;
							}
						}
						//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
						if (inf.ToString().StartsWith("attribute"))
						{
							NodeAttribute na = (NodeAttribute) inf;
							if (System.String.Compare(na.Name, "Variable label", false) == 0)
								nodeLabel = na.StringValue;
							if (System.String.Compare(na.Name, "State name", false) == 0)
							{
								//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
								states.Add(na.StringValue);
							}
						}
						//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
						if (inf.ToString().CompareTo("property") == 0)
						{
							// We know only three properties. Any other are ignored
							NodeAttribute na = (NodeAttribute) inf.jjtGetChild(0);
							if (System.String.Compare(na.StringValue, "nodetype", false) == 0)
							{
								nodeType = ((NodeAttribute) inf.jjtGetChild(1)).StringValue;
							}
							if (System.String.Compare(na.StringValue, "subtype", false) == 0)
							{
								subType = ((NodeAttribute) inf.jjtGetChild(1)).StringValue;
							}
							if (System.String.Compare(na.StringValue, "statevalue", false) == 0)
							{
								//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
								//UPGRADE_TODO: Format of parameters of constructor 'java.lang.Double.Double' are different in the equivalent in .NET. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1092"'
								stateValues.Add(System.Double.Parse(((NodeAttribute) inf.jjtGetChild(1)).StringValue));
							}
						}
					}
					// Now, create the node
					// If chance node
					if (nodeType.ToUpper().Equals("chance".ToUpper()))
					{
						DiscreteNode node = null;
						if (subType.ToUpper().Equals("numbered".ToUpper()))
							node = new NumberedLimidChanceNode(model);
						else
						{
							if (subType.ToUpper().Equals("boolean".ToUpper()))
								node = new BooleanLimidChanceNode(model);
							else
							{
								// The default option
								node = new LabeledLimidChanceNode(model);
							}
						}
						node.setLabel(nodeLabel);
						//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
						node.setNumberOfStates(states.Count);
						for (int k = 0; k < node.getNumberOfStates(); k++)
						{
							node.setStateLabel(k, (System.String) states[k]);
							//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
							if (node is NumberedLimidChanceNode && k < stateValues.Count)
								((NumberedLimidChanceNode) node).setStateValue(k, ((System.Double) stateValues[k]));
						}
						node.setPosition(new System.Drawing.Point(xPos, yPos));
						node.setName(nodeName);
					}
					// If decision node
					if (nodeType.ToUpper().Equals("decision".ToUpper()))
					{
						DiscreteNode node = null;
						if (subType.ToUpper().Equals("numbered".ToUpper()))
							node = new NumberedDiscreteDecisionChanceNode(model);
						else
						{
							if (subType.ToUpper().Equals("boolean".ToUpper()))
								node = new BooleanDiscreteDecisionChanceNode(model);
							else
							{
								// The default option
								node = new LabeledDiscreteDecisionChanceNode(model);
							}
						}
						node.setLabel(nodeLabel);
						//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
						node.setNumberOfStates(states.Count);
						for (int k = 0; k < node.getNumberOfStates(); k++)
						{
							node.setStateLabel(k, (System.String) states[k]);
							//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
							if (node is NumberedDiscreteDecisionChanceNode && k < stateValues.Count)
								((NumberedDiscreteDecisionChanceNode) node).setStateValue(k, ((System.Double) stateValues[k]));
						}
						node.setPosition(new System.Drawing.Point(xPos, yPos));
						node.setName(nodeName);
					}
					// If utility node
					if (nodeType.ToUpper().Equals("utility".ToUpper()))
					{
						TimeUtilityNode node = new TimeUtilityNode(model);
						node.setLabel(nodeLabel);
						node.setPosition(new System.Drawing.Point(xPos, yPos));
						node.setName(nodeName);
					}
				}
			}
			// The edge section(s)
			ite = structures.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				SimpleNode n = (SimpleNode) ite.Current;
				// This is a collection of edges
				for (int j = 0; j < n.jjtGetNumChildren(); j++)
				{
					SimpleNode arc = (SimpleNode) n.jjtGetChild(j);
					// This is a single edge (or a "member" declaration)
					//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					if (arc.ToString().ToUpper().Equals("arc".ToUpper()))
					{
						System.String parent = "";
						System.String child = "";
						NodeAttribute node1 = (NodeAttribute) arc.jjtGetChild(0);
						NodeAttribute node2 = (NodeAttribute) arc.jjtGetChild(1);
						if (node1.Name.ToUpper().Equals("parent".ToUpper()))
						{
							parent = node1.StringValue;
							child = node2.StringValue;
						}
						else
						{
							parent = node2.StringValue;
							child = node1.StringValue;
						}
						model.getNodeByName(child).addParent(model.getNodeByName(parent));
					}
				}
			}
			// The distribution section(s)
			ite = distributions.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				SimpleNode n = (SimpleNode) ite.Current;
				// This is a collection of distributions
				for (int j = 0; j < n.jjtGetNumChildren(); j++)
				{
					SimpleNode dist = (SimpleNode) n.jjtGetChild(j);
					// This is a single distribution
					System.String nodeName = "";
					//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
					System.Collections.ArrayList parents = new System.Collections.ArrayList();
					//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
					System.Collections.ArrayList indexes = new System.Collections.ArrayList();
					//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
					System.Collections.ArrayList values = new System.Collections.ArrayList();
					bool hasParents = false;
					for (int k = 0; k < dist.jjtGetNumChildren(); k++)
					{
						SimpleNode component = (SimpleNode) dist.jjtGetChild(k);
						// We only interprete statements that are supported
						// Is it a conditional distribution?
						//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
						if (component.ToString().Equals("condSet"))
						{
							hasParents = true;
							for (int l = 0; l < component.jjtGetNumChildren(); l++)
							{
								//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
								parents.Add(((NodeAttribute) component.jjtGetChild(l).jjtGetChild(0)).StringValue);
							}
						}
						//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
						if (component.ToString().Equals("privateDef"))
						{
							nodeName = ((NodeAttribute) component.jjtGetChild(0)).StringValue;
						}
						//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
						if (component.ToString().Equals("dpis"))
						{
							for (int l = 0; l < component.jjtGetNumChildren(); l++)
							{
								if (component.jjtGetChild(l) is IntListNode)
								{
									//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
									indexes.Add(((IntListNode) component.jjtGetChild(l)).Integers);
								}
								if (component.jjtGetChild(l) is FloatListNode)
								{
									//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
									values.Add(((FloatListNode) component.jjtGetChild(l)).Floats);
								}
							}
						}
					}
					Node node = model.getNodeByName(nodeName);
					if (node is DiscreteNode)
					{
						DefaultProbabilityPotential table = (DefaultProbabilityPotential) node.Table;
						if (hasParents)
						{
							//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
							for (int l = 0; l < indexes.Count; l++)
								table.setProbabilityProvider(table.convertToParentIndex((int[]) indexes[l]), new ArrayProbabilityProvider((float[]) values[l]));
						}
						else
						{
							table.setProbabilityProvider(0, new ArrayProbabilityProvider((float[]) values[0]));
						}
					}
					else
					{
						// This is temporary. We don't know the real size of the potential list
						PotentialTable table = (PotentialTable) node.Table;
						PotentialTable timeTable = (PotentialTable) ((TimeUtilityNode) node).TimeTable;
						if (hasParents)
						{
							//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
							for (int l = 0; l < indexes.Count; l++)
							{
								float[] val = (float[]) values[l];
								int[] ind = (int[]) indexes[l];
								int index = table.convertToIndex(ind);
								table.setDataItem(index, val[0]);
								timeTable.setDataItem(index, val[1]);
							}
						}
						else
						{
							table.setDataItem(0, ((float[]) values[0])[0]);
							timeTable.setDataItem(0, ((float[]) values[0])[1]);
						}
					}
				}
			}
			return model;
		}
	}
}