using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> A chance node where each state has a numerical interpretation.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// <p>Company: </p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class NumberedLimidChanceNode:LimidChanceNode, StateValues
	{
		
		/// <summary> A list of state values.</summary>
		//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		protected internal System.Collections.ArrayList values;
		
		/// <summary> Creates a numbered chance node within the specified model.</summary>
		/// <param name="ld">The model.
		/// @throws MLPDuplicateNodeNameException
		/// </param>
		public NumberedLimidChanceNode(LimidDomain ld):base(ld)
		{
			//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			values = new System.Collections.ArrayList();
		}
		
		/// <summary> Defines the state value for the specified state</summary>
		/// <param name="s">The state index.
		/// </param>
		/// <param name="v">The desired value.
		/// </param>
		public virtual void  setStateValue(int s, double v)
		{
			System.Object item;
			item = values[s];
			values[s] = v;
			System.Object generatedAux = item;
		}
		
		/// <summary> Returns the current value for the specified state</summary>
		/// <param name="s">The state index.
		/// </param>
		/// <returns> The current value.
		/// </returns>
		public virtual double getStateValue(int s)
		{
			return ((System.Double) values[s]);
		}
		
		/// <summary> Defines the number of states. The method is overridden because the number
		/// of state values must at all times match the number of state labels.
		/// </summary>
		/// <param name="i">The desired number of states.
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public override void  setNumberOfStates(int i)
		{
			if (i > this.getNumberOfStates())
			{
				for (int j = this.getNumberOfStates(); j < i; j++)
				{
					//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					values.Add(0.0);
				}
			}
			else
			{
				for (int j = i; j < this.getNumberOfStates(); j++)
				{
					//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.remove' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					values.RemoveAt(j);
				}
			}
			base.setNumberOfStates(i);
		}
		
		/// <summary> Returns the string "numbered".</summary>
		/// <returns> The string "numbered".
		/// </returns>
		protected internal override System.String getSubtype()
		{
			return "numbered";
		}
		
		//UPGRADE_TODO: The equivalent of method 'java.lang.Object.clone' is not an override method. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1143"'
		protected internal System.Object Clone()
		{
			try
			{
				NumberedLimidChanceNode newNode = new NumberedLimidChanceNode(this.HomeDomain);
				newNode.copyNode(this);
				for (int i = 0; i < this.getNumberOfStates(); i++)
					newNode.setStateValue(i, getStateValue(i));
				return newNode;
			}
			catch (MLPException ex)
			{
				return null;
			}
		}
		
		/// <summary> Returns true</summary>
		/// <returns> true
		/// </returns>
		public override bool hasStateValues()
		{
			return true;
		}
	}
}