using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> The default potential table of utility nodes and Limid potentials in cliques
	/// and separators.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class PotentialTable:AbstractPotential
	{
		override public double[] Data
		{
			get
			{
				return potentials;
			}
			
			set
			{
				if (value.Length == potentials.Length)
					potentials = value;
				else
					throw new System.ArgumentException("Size of array does not match size of table");
			}
			
		}
		override public int IdenticalValues
		{
			set
			{
				for (int i = 0; i < potentials.Length; i++)
					potentials[i] = value;
			}
			
		}
		private double[] potentials;
		
		/// <summary> Creates an empty potential table from a list of nodes defining the domain.</summary>
		/// <param name="nodes">A list of nodes.
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public PotentialTable(NodeList nodes):base(nodes)
		{
			potentials = new double[size];
		}
		
		/// <summary> Creates a new potential table by cloning an other. Please, observe that
		/// the two tables in some cases will share data.
		/// </summary>
		/// <param name="">source
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public PotentialTable(Potential source):this(source.Nodes)
		{
			this.Data = source.Data;
		}
		
		public override double getDataItem(int parm1)
		{
			return potentials[parm1];
		}
		
		public override double getDataItem(int[] state)
		{
			return this.getDataItem(this.convertToIndex(state));
		}
		
		public override void  setDataItem(int index, double value_Renamed)
		{
			if (index >= 0 && index < potentials.Length)
				potentials[index] = value_Renamed;
			else
				throw new System.ArgumentException("Specified index out of bounds");
		}
		
		public override void  setDataItem(int[] state, double value_Renamed)
		{
			this.setDataItem(this.convertToIndex(state), value_Renamed);
		}
		
		public override int[] convertParentIndexToStateArray(int index)
		{
			int[] state = new int[nodes.Count];
			int factor = 1;
			int dif = 0;
			for (int p = state.Length - 1; p > - 1; p--)
			{
				state[p] = ((index - dif) / factor) % (maximums[p] * factor);
				dif = dif + state[p] * factor;
				factor = maximums[p] * factor;
			}
			return state;
		}
	}
}