using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> The result of a single iteration of the Single Policy Updating algorithm.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class SPURecord
	{
		/// <summary> Returns all utility values.</summary>
		/// <returns> An array of utility values.
		/// </returns>
		virtual internal double[] AllValues
		{
			get
			{
				return values;
			}
			
		}
		private double utility;
		private double[] values;
		
		/// <summary> Creates a record based on the specified number of decision nodes of the
		/// model.
		/// </summary>
		/// <param name="decisionNodeCount">The total number of decision nodes in the model.
		/// </param>
		public SPURecord(int decisionNodeCount)
		{
			values = new double[decisionNodeCount];
		}
		
		/// <summary> Defines the utility after the iteration has been performed.</summary>
		/// <param name="u">The utility result.
		/// </param>
		internal virtual void  setUtility(double u)
		{
			utility = u;
		}
		
		/// <summary> Returns the utility result.</summary>
		/// <returns> The utility result.
		/// </returns>
		internal double getUtility()
		{
			return utility;
		}
		
		/// <summary> Defines the utility after optimizing the specified decision.</summary>
		/// <param name="decision">The decision.
		/// </param>
		/// <param name="value">The corresponding utility value.
		/// </param>
		internal virtual void  setValue(int decision, double value_Renamed)
		{
			values[decision] = value_Renamed;
		}
		
		/// <summary> Returns the utility value corresponding to a decision.</summary>
		/// <param name="decision">The decision.
		/// </param>
		/// <returns> The corresponding utility value.
		/// </returns>
		internal double getDecisionValue(int decision)
		{
			return values[decision];
		}
	}
}