using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> An object representing the result of a full Single Policy Updating operation.
	/// It contains a list of elements of type SPURecord.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	/// <seealso cref="SPURecord">
	/// </seealso>
	public class SPUResult
	{
		/// <summary> Returns the number of records.</summary>
		/// <returns> The number of records.
		/// </returns>
		virtual public int Size
		{
			get
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				return records.Count;
			}
			
		}
		/// <summary> Returns all records as an array.</summary>
		/// <returns> An array of records.
		/// </returns>
		virtual public SPURecord[] AllRecords
		{
			get
			{
				SPURecord[] array = new SPURecord[Size];
				return (SPURecord[]) SupportClass.ArrayListSupport.ToArray(records, array);
			}
			
		}
		/// <summary> Returns a vector of coloumn names to be used for showing the result in a
		/// table. A Vector is returned (istead of an ArrayList) because the
		/// javax.swing.table.DefaultTableModel used for showing the results needs
		/// a Vector as input.
		/// </summary>
		/// <returns> A vector of String
		/// </returns>
		/// <seealso cref="javax.swing.JTable">
		/// </seealso>
		virtual public System.Collections.ArrayList ColumnNames
		{
			get
			{
				System.Collections.ArrayList names = new System.Collections.ArrayList(10);
				names.Add("Decision");
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				for (int i = 0; i < records.Count; i++)
				{
					names.Add("Iteration " + (i + 1));
				}
				return names;
			}
			
		}
		/// <summary> Returns a vector of rows contaning the data from the result object. The
		/// purpose is to generate the data for a table. A Vector is returned (istead
		/// of an ArrayList) because the javax.swing.table.DefaultTableModel used for
		/// showing the results needs a Vector as input.
		/// </summary>
		/// <returns> A vector rows (each row is also a vector).
		/// </returns>
		/// <seealso cref="javax.swing.JTable">
		/// </seealso>
		virtual public System.Collections.ArrayList Rows
		{
			get
			{
				System.Collections.ArrayList rows = new System.Collections.ArrayList(10);
				for (int i = 0; i < decisionNodes.Count; i++)
				{
					System.Collections.ArrayList aRow = new System.Collections.ArrayList(10);
					aRow.Add(decisionNodes[i]);
					//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					for (int j = 0; j < records.Count; j++)
					{
						aRow.Add(((SPURecord) records[j]).getDecisionValue(i));
					}
					rows.Add(aRow);
				}
				return rows;
			}
			
		}
		//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		private System.Collections.ArrayList records;
		private NodeList decisionNodes;
		
		/// <summary> Creates a new empty result object from a list of decision nodes.</summary>
		/// <param name="decNodes">The list of decision nodes.
		/// </param>
		public SPUResult(NodeList decNodes)
		{
			//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			records = new System.Collections.ArrayList();
			decisionNodes = decNodes;
		}
		
		/// <summary> Adds a new record.</summary>
		/// <param name="record">The new record.
		/// </param>
		public virtual void  addRecord(SPURecord record)
		{
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			records.Add(record);
		}
		
		
		/// <summary> Returns a specified record.</summary>
		/// <param name="index">The index of the desired record.
		/// </param>
		/// <returns> The record.
		/// </returns>
		public virtual SPURecord getRecord(int index)
		{
			return ((SPURecord) records[index]);
		}
	}
}