using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A command adding an edge to the active Limid.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class AddEdgeCommand : Command
	{
		private Node parent;
		private Node child;
		private LimidCanvas canvas;
		
		/// <summary> Creates a command object.</summary>
		/// <param name="parent">The parent node
		/// </param>
		/// <param name="child">The child node
		/// </param>
		/// <param name="can">The active canvas
		/// </param>
		public AddEdgeCommand(Node parent, Node child, LimidCanvas can)
		{
			this.parent = parent;
			this.child = child;
			canvas = can;
		}
		
		public virtual void  execute()
		{
			try
			{
				child.addParent(parent);
				canvas.Refresh();
			}
			catch (MLPException ex)
			{
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.getMessage' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.ERROR_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, ex.Message, "Illegal edge error", JOptionPane.ERROR_MESSAGE);
			}
		}
		
		public virtual void  undo()
		{
			try
			{
				child.removeParent(parent);
				canvas.Refresh();
			}
			catch (MLPException ex)
			{
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.getMessage' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.ERROR_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, ex.Message, "Illegal operation", JOptionPane.ERROR_MESSAGE);
			}
		}
		
		public virtual System.String ToString()
		{
			return "Add egde";
		}
	}
}