using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A command adding a node to the active Limid.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class AddNodeCommand : Command
	{
		private Node node;
		private LimidCanvas canvas;
		private bool addChanceMode;
		private bool addDecisionMode;
		private bool addUtilityMode;
		private System.Drawing.Point point;
		
		/// <summary> Creates a command object.</summary>
		/// <param name="addC">true if a chance node should be added
		/// </param>
		/// <param name="addD">true if a decision node should be added
		/// </param>
		/// <param name="addU">true if a utility node should be added
		/// </param>
		/// <param name="p">The position of the node
		/// </param>
		/// <param name="can">The active canvas
		/// </param>
		public AddNodeCommand(bool addC, bool addD, bool addU, System.Drawing.Point p, LimidCanvas can)
		{
			addChanceMode = addC;
			addDecisionMode = addD;
			addUtilityMode = addU;
			canvas = can;
			point = p;
		}
		
		public virtual void  execute()
		{
			try
			{
				LimidDomain limid = canvas.Limid;
				node = null;
				if (addChanceMode)
					node = new LimidChanceNode(limid);
				if (addDecisionMode)
					node = new DiscreteDecisionChanceNode(limid);
				if (addUtilityMode)
					node = new TimeUtilityNode(limid);
				if (node != null)
					node.setPosition(point);
				canvas.Refresh();
			}
			catch (MLPDuplicateNodeNameException ex)
			{
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.getMessage' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.ERROR_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, ex.Message, "The node name already exists", JOptionPane.ERROR_MESSAGE);
			}
		}
		
		public virtual void  undo()
		{
			LimidDomain limid = canvas.Limid;
			try
			{
				if (node != null)
					limid.removeNode(node);
				canvas.Refresh();
			}
			catch (MLPOperationNotAllowedException ex)
			{
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.getMessage' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.ERROR_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, ex.Message, "The operation is not allowed", JOptionPane.ERROR_MESSAGE);
			}
		}
		
		public virtual System.String ToString()
		{
			return "Add node";
		}
	}
}