using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A class representing the command history of the frame.  It remembers the
	/// 10 most recent commands. The "Redo" option is cancelled if the user deviates
	/// from the history (for instance by calling unDo 5 times followed by 2 calls
	/// to reDo and then does something else).
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class CommandHistory
	{
		private SupportClass.ListCollectionSupport commands;
		private int pointer;
		//UPGRADE_TODO: Class 'javax.swing.JMenuItem' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		private JMenuItem unDoMenu;
		//UPGRADE_TODO: Class 'javax.swing.JMenuItem' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		private JMenuItem reDoMenu;
		
		/// <summary> Creates a history object.</summary>
		/// <param name="unDo">The menu item representing "Undo"
		/// </param>
		/// <param name="reDo">The menu item representing "Redo"
		/// </param>
		//UPGRADE_TODO: Class 'javax.swing.JMenuItem' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public CommandHistory(JMenuItem unDo, JMenuItem reDo)
		{
			commands = new SupportClass.ListCollectionSupport();
			unDoMenu = unDo;
			reDoMenu = reDo;
			pointer = - 1;
		}
		
		/// <summary> Adds a command to the history. The command is executed when added, so it
		/// should not be executed before adding.
		/// </summary>
		/// <param name="com">The command to be added.
		/// </param>
		public virtual void  addCommand(Command com)
		{
			if (hasNext())
			{
				for (int i = pointer + 1; i < commands.Count; i++)
					commands.RemoveElement(i);
				//UPGRADE_TODO: Method 'javax.swing.JMenuItem.setEnabled' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				reDoMenu.setEnabled(false);
			}
			com.execute();
			commands.Add(com);
			// We decide to keep the 10 most recent actions. The value is arbitrary.
			if (commands.Count > 10)
				commands.RemoveFirst();
			pointer = commands.Count - 1;
			//UPGRADE_TODO: Method 'javax.swing.AbstractButton.setText' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			unDoMenu.setText("Undo " + com);
			//UPGRADE_TODO: Method 'javax.swing.JMenuItem.setEnabled' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			unDoMenu.setEnabled(true);
		}
		
		/// <summary> Checks whether more "Undo"-commands are possible.</summary>
		/// <returns> true if there are more
		/// </returns>
		public virtual bool hasPrevious()
		{
			return (pointer > - 1);
		}
		
		/// <summary> Checks whether more "Redo"-commands are possible</summary>
		/// <returns> true if there are more
		/// </returns>
		public virtual bool hasNext()
		{
			return (pointer < commands.Count - 1);
		}
		
		/// <summary> Undo the previous command in the history. A pointer is maintained by the
		/// object.
		/// </summary>
		public virtual void  unDo()
		{
			if (hasPrevious())
			{
				Command com = (Command) commands[pointer];
				com.undo();
				pointer--;
				//UPGRADE_TODO: Method 'javax.swing.AbstractButton.setText' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				reDoMenu.setText("Redo " + com);
				//UPGRADE_TODO: Method 'javax.swing.JMenuItem.setEnabled' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				reDoMenu.setEnabled(true);
				if (hasPrevious())
				{
					com = (Command) commands[pointer];
					//UPGRADE_TODO: Method 'javax.swing.AbstractButton.setText' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					unDoMenu.setText("Undo " + com);
					//UPGRADE_TODO: Method 'javax.swing.JMenuItem.setEnabled' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					unDoMenu.setEnabled(true);
				}
				else
				{
					//UPGRADE_TODO: Method 'javax.swing.JMenuItem.setEnabled' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					unDoMenu.setEnabled(false);
				}
			}
		}
		
		/// <summary> Redo the next command in the history. A pointer is maintained by the object</summary>
		public virtual void  reDo()
		{
			if (hasNext())
			{
				Command com = (Command) commands[pointer + 1];
				com.execute();
				pointer++;
				//UPGRADE_TODO: Method 'javax.swing.AbstractButton.setText' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				unDoMenu.setText("Undo " + com);
				//UPGRADE_TODO: Method 'javax.swing.JMenuItem.setEnabled' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				unDoMenu.setEnabled(true);
				if (hasNext())
				{
					com = (Command) commands[pointer + 1];
					//UPGRADE_TODO: Method 'javax.swing.AbstractButton.setText' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					reDoMenu.setText("Redo " + com);
					//UPGRADE_TODO: Method 'javax.swing.JMenuItem.setEnabled' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					reDoMenu.setEnabled(true);
				}
				else
				{
					//UPGRADE_TODO: Method 'javax.swing.JMenuItem.setEnabled' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					reDoMenu.setEnabled(false);
				}
			}
		}
	}
}