using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A command copying a selection. The selection may be a single node or all
	/// nodes in an area.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class CopyCommand : Command
	{
		private SelectionSet selection;
		private SelectionSet oldSelection;
		private LimidCanvas canvas;
		private LimidDomain oldContainer;
		
		/// <summary> Creates a command object.</summary>
		/// <param name="sel">The active selection.
		/// </param>
		/// <param name="can">The active canvas.
		/// </param>
		public CopyCommand(SelectionSet sel, LimidCanvas can)
		{
			oldSelection = new SelectionSet(sel);
			selection = sel;
			canvas = can;
		}
		
		public virtual void  execute()
		{
			oldContainer = LimidFrame.nodeContainer;
			LimidFrame.nodeContainer = new LimidDomain();
			selection.Clear();
			selection.addAll(canvas.SelectedItems);
		}
		
		public virtual void  undo()
		{
			selection.addAll(oldSelection);
			LimidFrame.nodeContainer = oldContainer;
		}
		
		public virtual System.String ToString()
		{
			return "Copy";
		}
	}
}