using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> An editor for decisions. The editor uses a JComboBox as component.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	//UPGRADE_TODO: Interface 'javax.swing.table.TableCellEditor' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
	public class DecisionCellEditor : TableCellEditor
	{
		virtual public System.Object CellEditorValue
		{
			get
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Method 'javax.swing.JComboBox.getSelectedItem' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				return combo.getSelectedItem().ToString();
			}
			
		}
		//UPGRADE_TODO: Class 'javax.swing.JComboBox' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		internal JComboBox combo;
		internal SupportClass.ListCollectionSupport listeners;
		
		/// <summary> Creates an instance of the editor.</summary>
		/// <param name="node">The decision node in question.
		/// </param>
		public DecisionCellEditor(DiscreteNode node)
		{
			System.String[] names = new System.String[node.getNumberOfStates()];
			for (int i = 0; i < names.Length; i++)
				names[i] = node.getStateLabel(i);
			//UPGRADE_TODO: Constructor 'javax.swing.JComboBox.JComboBox' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			combo = new JComboBox(names);
			listeners = new SupportClass.ListCollectionSupport();
		}
		
		//UPGRADE_TODO: Class 'javax.swing.JTable' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual System.Windows.Forms.Control getTableCellEditorComponent(JTable table, System.Object value_Renamed, bool isSelected, int row, int column)
		{
			bool notFound = true;
			int i = 0;
			//UPGRADE_TODO: Method 'javax.swing.JComboBox.getItemCount' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			while (notFound && i < combo.getItemCount())
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Method 'javax.swing.JComboBox.getItemAt' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				if (value_Renamed.ToString().CompareTo(combo.getItemAt(i).ToString()) == 0)
				{
					//UPGRADE_TODO: Method 'javax.swing.JComboBox.setSelectedIndex' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					combo.setSelectedIndex(i);
					notFound = false;
				}
				i++;
			}
			return combo;
		}
		
		/// <summary> Two mouse clicks activates the editor.</summary>
		/// <param name="">anEvent
		/// </param>
		/// <returns> true if anEvent is a MouseEvent with a click count higher than 1
		/// </returns>
		//UPGRADE_ISSUE: Class 'java.util.EventObject' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1000_javautilEventObject"'
		public virtual bool isCellEditable(EventObject anEvent)
		{
			if (anEvent is System.Windows.Forms.MouseEventArgs)
			{
				System.Windows.Forms.MouseEventArgs ev = (System.Windows.Forms.MouseEventArgs) anEvent;
				//UPGRADE_ISSUE: Method 'java.awt.event.MouseEvent.getClickCount' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1000"'
				if (ev.getClickCount() > 1)
					return true;
			}
			return false;
		}
		
		/// <summary> Returns true</summary>
		/// <param name="anEvent">Any event
		/// </param>
		/// <returns> true
		/// </returns>
		//UPGRADE_ISSUE: Class 'java.util.EventObject' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1000_javautilEventObject"'
		public virtual bool shouldSelectCell(EventObject anEvent)
		{
			return true;
		}
		
		public virtual bool stopCellEditing()
		{
			SupportClass.ListCollectionSupport list = SupportClass.CreateCollection(listeners);
			System.Collections.IEnumerator ite = list.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'javax.swing.event.CellEditorListener.editingStopped' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Interface 'javax.swing.event.CellEditorListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				//UPGRADE_TODO: Constructor 'javax.swing.event.ChangeEvent.ChangeEvent' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				((CellEditorListener) ite.Current).editingStopped(new ChangeEvent(this));
			}
			return true;
		}
		
		public virtual void  cancelCellEditing()
		{
			SupportClass.ListCollectionSupport list = SupportClass.CreateCollection(listeners);
			System.Collections.IEnumerator ite = list.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'javax.swing.event.CellEditorListener.editingCanceled' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Interface 'javax.swing.event.CellEditorListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				//UPGRADE_TODO: Constructor 'javax.swing.event.ChangeEvent.ChangeEvent' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				((CellEditorListener) ite.Current).editingCanceled(new ChangeEvent(this));
			}
		}
		
		//UPGRADE_TODO: Interface 'javax.swing.event.CellEditorListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual void  addCellEditorListener(CellEditorListener l)
		{
			listeners.Add(l);
		}
		
		//UPGRADE_TODO: Interface 'javax.swing.event.CellEditorListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual void  removeCellEditorListener(CellEditorListener l)
		{
			listeners.RemoveElement(l);
		}
	}
}