using System;
using dk.esthauge.limid.api;
using dk.kvl.ihh.prodstyr.mlhmp;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> The table model used for editing the potential of a decision node.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class DecisionTableModel : CommonProbabilityTableModel
	{
		virtual public int RowCount
		{
			get
			{
				return rowCount;
			}
			
		}
		virtual public int ColumnCount
		{
			get
			{
				return 1;
			}
			
		}
		virtual public DefaultProbabilityPotential ProbabilityTable
		{
			get
			{
				return table;
			}
			
		}
		//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		virtual public System.Collections.ArrayList AllPotentials
		{
			get
			{
				//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
				System.Collections.ArrayList pot = new System.Collections.ArrayList();
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				pot.Add(table);
				return pot;
			}
			
		}
		private DiscreteDecisionChanceNode node;
		private DefaultProbabilityPotential table;
		private SupportClass.ListCollectionSupport listeners;
		private int rowCount;
		
		/// <summary> Creates a table model from a decision node.</summary>
		/// <param name="node">The decision node.
		/// </param>
		public DecisionTableModel(DiscreteDecisionChanceNode node)
		{
			this.node = node;
			// Make a copy of the table and edit the copy
			try
			{
				table = new DefaultProbabilityPotential(node.Table.Nodes);
				// Elegant, but inefficient, source code
				table.Data = node.Table.Data;
			}
			catch (MLPOperationNotAllowedException ex)
			{
				// This can never happen since a table for this node already exists
			}
			rowCount = table.ParentStateCount;
			listeners = new SupportClass.ListCollectionSupport();
		}
		
		public virtual System.String getColumnName(int columnIndex)
		{
			return "Decision";
		}
		
		public virtual System.Type getColumnClass(int columnIndex)
		{
			return "".GetType();
		}
		
		public virtual bool isCellEditable(int rowIndex, int columnIndex)
		{
			return true;
		}
		
		public virtual System.Object getValueAt(int rowIndex, int columnIndex)
		{
			ProbabilityProvider pp = table.getProbabilityProvider(rowIndex);
			bool notFound = true;
			int i = 0;
			while (notFound && i < table.StateCount)
			{
				if (System.Math.abs(pp.get_Renamed(i) - 1) < 0.00001F)
				{
					notFound = false;
					return node.getStateLabel(i);
				}
				i++;
			}
			return "Not found";
		}
		
		public virtual void  setValueAt(System.Object aValue, int rowIndex, int columnIndex)
		{
			//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			System.String value_Renamed = aValue.ToString();
			ProbabilityProvider pp = table.getProbabilityProvider(rowIndex);
			bool notFound = true;
			int i = 0;
			while (notFound && i < table.StateCount)
			{
				if (node.getStateLabel(i).CompareTo(value_Renamed) == 0)
				{
					pp.set_Renamed(i, 1F);
					notFound = false;
				}
				else
					pp.set_Renamed(i, 0F);
				i++;
			}
			//UPGRADE_TODO: Class 'javax.swing.event.TableModelEvent' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Constructor 'javax.swing.event.TableModelEvent.TableModelEvent' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Field 'javax.swing.event.TableModelEvent.UPDATE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			TableModelEvent event_Renamed = new TableModelEvent(this, rowIndex, rowIndex, columnIndex, TableModelEvent.UPDATE);
			System.Collections.IEnumerator ite = listeners.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'javax.swing.event.TableModelListener.tableChanged' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				((TableModelListener) ite.Current).tableChanged(event_Renamed);
			}
		}
		
		//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual void  addTableModelListener(TableModelListener l)
		{
			listeners.Add(l);
		}
		
		//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual void  removeTableModelListener(TableModelListener l)
		{
			listeners.RemoveElement(l);
		}
		
		public virtual double getRowSum(int row)
		{
			double sum = 0;
			//UPGRADE_TODO: Method 'javax.swing.table.TableModel.getColumnCount' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			for (int i = 0; i < this.getColumnCount(); i++)
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Float.floatValue' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Method 'javax.swing.table.TableModel.getValueAt' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				sum = sum + (float) ((System.Single) this.getValueAt(row, i));
			}
			return sum;
		}
	}
}