using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A command deleting selected items.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// <p>Company: </p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class DeleteCommand : Command
	{
		private void  InitBlock()
		{
			allRemovedItems = new SupportClass.ListCollectionSupport();
		}
		private LimidCanvas canvas;
		private SupportClass.ListCollectionSupport selection;
		//UPGRADE_NOTE: The initialization of  'allRemovedItems' was moved to method 'InitBlock'. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1005"'
		private SupportClass.ListCollectionSupport allRemovedItems;
		
		/// <summary> Creates a command object.</summary>
		/// <param name="can">The active canvas. The current selection is deleted.
		/// </param>
		public DeleteCommand(LimidCanvas can)
		{
			InitBlock();
			canvas = can;
			selection = canvas.SelectedItems;
		}
		
		public virtual void  execute()
		{
			LimidDomain activeLimid = canvas.Limid;
			//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showConfirmDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Field 'javax.swing.JOptionPane.YES_NO_OPTION' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Field 'javax.swing.JOptionPane.QUESTION_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			int answer = JOptionPane.showConfirmDialog(canvas.Owner, "Do you really want to delete the selected item(s)?", "Please confirm", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
			if (answer == 0)
			{
				System.Collections.IEnumerator ite = selection.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Drawable shape = (Drawable) ite.Current;
					// It is enough to delete the nodes
					if (shape is Node)
					{
						Node node = (Node) shape;
						try
						{
							allRemovedItems.Add(node);
							NodeList parents = node.Parents;
							System.Collections.IEnumerator jte = parents.GetEnumerator();
							//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
							while (jte.MoveNext())
							{
								//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
								Node parent = (Node) jte.Current;
								//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
								DirectedEdge edge = new DirectedEdge(parent, node, node.HomeDomain.getNodeSize().getWidth() / 2);
								allRemovedItems.Add(edge);
							}
							NodeList children = node.Children;
							jte = children.GetEnumerator();
							//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
							while (jte.MoveNext())
							{
								//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
								Node child = (Node) jte.Current;
								//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
								DirectedEdge edge = new DirectedEdge(node, child, node.HomeDomain.getNodeSize().getWidth() / 2);
								allRemovedItems.Add(edge);
							}
							activeLimid.removeNode(node);
						}
						catch (MLPOperationNotAllowedException ex)
						{
							//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
							//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
							//UPGRADE_TODO: Field 'javax.swing.JOptionPane.ERROR_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
							JOptionPane.showMessageDialog(canvas.Owner, "Error deleting node \"" + node + "\". Not allowed", "Operation not allowed", JOptionPane.ERROR_MESSAGE);
						}
					}
				}
				canvas.clearSelection();
			}
		}
		
		public virtual void  undo()
		{
			try
			{
				// First, re-add removed nodes
				System.Collections.IEnumerator ite = allRemovedItems.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Drawable shape = (Drawable) ite.Current;
					if (shape is Node)
					{
						Node node = (Node) shape;
						canvas.Limid.addNode(node);
					}
				}
				// Then, re-add removed edges
				ite = allRemovedItems.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Drawable shape = (Drawable) ite.Current;
					if (shape is DirectedEdge)
					{
						DirectedEdge edge = (DirectedEdge) shape;
						Node child = edge.Owner;
						Node parent = edge.Neighbor;
						child.addParent(parent);
					}
				}
				canvas.SelectionSet.AddAll(selection);
				canvas.Refresh();
			}
			catch (MLPException ex)
			{
				// This should never happen
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.getMessage' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.ERROR_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, ex.Message, "Operation not allowed", JOptionPane.ERROR_MESSAGE);
			}
		}
		
		public virtual System.String ToString()
		{
			return "Delete";
		}
	}
}