using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A command that deletes an edge.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class DeleteEdgeCommand : Command
	{
		private LimidCanvas canvas;
		private SupportClass.ListCollectionSupport selected;
		private Edge removedEdge;
		
		/// <summary> Creates a command object</summary>
		/// <param name="can">The active canvas. The selected edge is deleted.
		/// </param>
		public DeleteEdgeCommand(LimidCanvas can)
		{
			canvas = can;
			selected = canvas.SelectedItems;
		}
		
		public virtual void  execute()
		{
			LimidDomain activeLimid = canvas.Limid;
			Drawable shape = (Drawable) selected[0];
			try
			{
				if (selected.Count == 1 && shape is Edge)
				{
					removedEdge = (Edge) shape;
					System.String message;
					//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showConfirmDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					//UPGRADE_TODO: Field 'javax.swing.JOptionPane.YES_NO_OPTION' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					//UPGRADE_TODO: Field 'javax.swing.JOptionPane.QUESTION_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					int answer = JOptionPane.showConfirmDialog(canvas.Owner, "OK to delete selected edge?", "Confirm deleting of edge", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
					if (answer == 0)
					{
						Node child = removedEdge.Owner;
						Node parent = removedEdge.Neighbor;
						child.removeParent(parent);
						canvas.clearSelection();
					}
				}
			}
			catch (MLPOperationNotAllowedException ex)
			{
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.getMessage' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.ERROR_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, ex.Message, "Operation not allowed", JOptionPane.ERROR_MESSAGE);
			}
			canvas.Refresh();
		}
		
		public virtual void  undo()
		{
			if (removedEdge != null)
			{
				try
				{
					Node child = removedEdge.Owner;
					Node parent = removedEdge.Neighbor;
					child.addParent(parent);
					canvas.SelectionSet.AddAll(selected);
					canvas.Refresh();
				}
				catch (MLPOperationNotAllowedException ex)
				{
					//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.getMessage' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					//UPGRADE_TODO: Field 'javax.swing.JOptionPane.ERROR_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					JOptionPane.showMessageDialog(canvas.Owner, ex.Message, "Operation not allowed", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
		
		public virtual System.String ToString()
		{
			return "Delete edge";
		}
	}
}