using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A command changing the properties of a node.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class EditNodeCommand : Command
	{
		private LimidCanvas canvas;
		private Node node;
		private DiscreteNode discreteNode = null;
		private System.String name;
		private System.String label;
		//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		private System.Collections.ArrayList stateLabels;
		private bool firstTime = true;
		
		/// <summary> Creates a command object.</summary>
		/// <param name="can">The active canvas.
		/// </param>
		public EditNodeCommand(LimidCanvas can)
		{
			canvas = can;
			NodeList nodes = canvas.SelectedNodes;
			node = (Node) nodes[0];
			if (node is DiscreteNode)
				discreteNode = (DiscreteNode) node;
			name = node.getName();
			label = node.getLabel();
			if (discreteNode != null)
			{
				int count = discreteNode.getNumberOfStates();
				//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
				stateLabels = new System.Collections.ArrayList(count);
				for (int i = 0; i < count; i++)
				{
					//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					stateLabels.Add(discreteNode.getStateLabel(i));
				}
			}
		}
		
		public virtual void  execute()
		{
			if (!firstTime)
			{
				System.String newName = node.getName();
				System.String newLabel = node.getLabel();
				//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
				System.Collections.ArrayList newStateLabels = new System.Collections.ArrayList();
				if (discreteNode != null)
				{
					int newCount = discreteNode.getNumberOfStates();
					//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
					newStateLabels = new System.Collections.ArrayList(newCount);
					for (int i = 0; i < newCount; i++)
					{
						//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
						newStateLabels.Add(discreteNode.getStateLabel(i));
					}
				}
				try
				{
					node.setName(name);
				}
				catch (MLPDuplicateNodeNameException ex)
				{
					// Will never happen
					//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					//UPGRADE_TODO: Field 'javax.swing.JOptionPane.WARNING_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					JOptionPane.showMessageDialog(canvas.Owner, "An unexpected error " + "was detected. Please inform the " + "developer. The exception thrown was:\n" + ex.ToString() + "\nYou should save your work!", "Software BUG?", JOptionPane.WARNING_MESSAGE);
				}
				node.setLabel(label);
				if (discreteNode != null)
				{
					try
					{
						//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
						discreteNode.setNumberOfStates(stateLabels.Count);
					}
					catch (MLPOperationNotAllowedException ex)
					{
						// Will never happen
						//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
						//UPGRADE_TODO: Field 'javax.swing.JOptionPane.WARNING_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
						JOptionPane.showMessageDialog(canvas.Owner, "An unexpected error " + "was detected. Please inform the " + "developer. The exception thrown was:\n" + ex.ToString() + "\nYou should save your work!", "Software BUG?", JOptionPane.WARNING_MESSAGE);
					}
					//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					for (int i = 0; i < stateLabels.Count; i++)
						discreteNode.setStateLabel(i, (System.String) stateLabels[i]);
				}
				// Finally, remember the new values
				name = newName;
				label = newLabel;
				if (discreteNode != null)
					stateLabels = newStateLabels;
			}
			//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			NodeEditor editor = new NodeEditor(canvas.Owner, "Edit properties for node " + node, true, node);
			//UPGRADE_TODO: Method 'java.awt.Component.setVisible' was converted to 'System.Windows.Forms.Control.Visible' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073"'
			//UPGRADE_TODO: System.Windows.Forms.Application.Run must be called to start a main form. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1135"'
			editor.Visible = true;
			canvas.Refresh();
			firstTime = false;
		}
		
		public virtual void  undo()
		{
			// Remember the present values for re-do
			System.String newName = node.getName();
			System.String newLabel = node.getLabel();
			//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			System.Collections.ArrayList newStateLabels = new System.Collections.ArrayList();
			if (discreteNode != null)
			{
				int newCount = discreteNode.getNumberOfStates();
				//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
				newStateLabels = new System.Collections.ArrayList(newCount);
				for (int i = 0; i < newCount; i++)
				{
					//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					newStateLabels.Add(discreteNode.getStateLabel(i));
				}
			}
			// Re-set to old values
			try
			{
				node.setName(name);
			}
			catch (MLPDuplicateNodeNameException ex)
			{
				// Will never happen
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.WARNING_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, "An unexpected error " + "was detected. Please inform the " + "developer. The exception thrown was:\n" + ex.ToString() + "\nYou should save your work!", "Software BUG?", JOptionPane.WARNING_MESSAGE);
			}
			node.setLabel(label);
			if (discreteNode != null)
			{
				try
				{
					//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					discreteNode.setNumberOfStates(stateLabels.Count);
				}
				catch (MLPOperationNotAllowedException ex)
				{
					// Will never happen
					//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					//UPGRADE_TODO: Field 'javax.swing.JOptionPane.WARNING_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					JOptionPane.showMessageDialog(canvas.Owner, "An unexpected error " + "was detected. Please inform the " + "developer. The exception thrown was:\n" + ex.ToString() + "\nYou should save your work!", "Software BUG?", JOptionPane.WARNING_MESSAGE);
				}
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				for (int i = 0; i < stateLabels.Count; i++)
					discreteNode.setStateLabel(i, (System.String) stateLabels[i]);
			}
			canvas.Refresh();
			// Finally, remember the new values
			name = newName;
			label = newLabel;
			if (discreteNode != null)
				stateLabels = newStateLabels;
		}
		
		
		public virtual System.String ToString()
		{
			return "Edit discrete node";
		}
	}
}