using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A command that relocates the selected nodes.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// <p>Company: </p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class MoveCommand : Command
	{
		private LimidCanvas canvas;
		private int xMove;
		private int yMove;
		private bool first = true;
		
		/// <summary> Creates a command object</summary>
		/// <param name="from">The original position
		/// </param>
		/// <param name="to">The destination
		/// </param>
		/// <param name="can">The active canvas. The currently selected items are moved.
		/// </param>
		public MoveCommand(System.Drawing.Point from, System.Drawing.Point to, LimidCanvas can)
		{
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.Point.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			xMove = (int) (from.getX() - to.getX());
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.Point.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			yMove = (int) (from.getY() - to.getY());
			canvas = can;
		}
		
		public virtual void  execute()
		{
			if (!first)
			{
				System.Collections.IEnumerator ite = canvas.SelectedNodes.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node node = (Node) ite.Current;
					//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					Point2D oldPos = node.getPosition();
					//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					Point2D newPos = new System.Drawing.Point((int) (oldPos.getX() - xMove), (int) (oldPos.getY() - yMove));
					node.setPosition(newPos);
				}
				canvas.Refresh();
			}
			first = false;
		}
		
		public virtual void  undo()
		{
			System.Collections.IEnumerator ite = canvas.SelectedNodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				Point2D oldPos = node.getPosition();
				//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				Point2D newPos = new System.Drawing.Point((int) (oldPos.getX() + xMove), (int) (oldPos.getY() + yMove));
				node.setPosition(newPos);
			}
			canvas.Refresh();
		}
		
		public virtual System.String ToString()
		{
			return "Move selected items";
		}
	}
}