using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A command changing the network properties (name, node size, colors).
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// <p>Company: </p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class NetworkPropertyCommand : Command
	{
		private LimidDomain limid;
		private LimidCanvas canvas;
		private System.String oldName;
		private System.String newName;
		private int oldSize;
		private int newSize;
		private System.Drawing.Color oldCColor;
		private System.Drawing.Color newCColor;
		private System.Drawing.Color oldDColor;
		private System.Drawing.Color newDColor;
		private System.Drawing.Color oldUColor;
		private System.Drawing.Color newUColor;
		
		/// <summary> Creates a command object</summary>
		/// <param name="can">The active canvas
		/// </param>
		public NetworkPropertyCommand(LimidCanvas can)
		{
			canvas = can;
			limid = canvas.Limid;
			oldName = limid.Name;
			oldSize = limid.getNodeSize().Width;
			oldCColor = limid.ChanceNodeColor;
			oldDColor = limid.DecisionNodeColor;
			oldUColor = limid.UtilityNodeColor;
			NetworkPropertiesDialog dialog = new NetworkPropertiesDialog(canvas.Owner, "Edit network properties", true, limid);
			//UPGRADE_TODO: Method 'java.awt.Component.setVisible' was converted to 'System.Windows.Forms.Control.Visible' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073"'
			//UPGRADE_TODO: System.Windows.Forms.Application.Run must be called to start a main form. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1135"'
			dialog.Visible = true;
			newName = limid.Name;
			newSize = limid.getNodeSize().Width;
			newCColor = limid.ChanceNodeColor;
			newDColor = limid.DecisionNodeColor;
			newUColor = limid.UtilityNodeColor;
		}
		
		public virtual void  execute()
		{
			limid.Name = newName;
			limid.setNodeSize(newSize);
			limid.ChanceNodeColor = newCColor;
			limid.DecisionNodeColor = newDColor;
			limid.UtilityNodeColor = newUColor;
			//UPGRADE_TODO: Method 'javax.swing.JTabbedPane.setTitleAt' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'javax.swing.JTabbedPane.getSelectedIndex' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			canvas.Owner.tabbedPane.setTitleAt(canvas.Owner.tabbedPane.getSelectedIndex(), limid.Name);
			canvas.Refresh();
		}
		
		public virtual void  undo()
		{
			limid.Name = oldName;
			limid.setNodeSize(oldSize);
			limid.ChanceNodeColor = oldCColor;
			limid.DecisionNodeColor = oldDColor;
			limid.UtilityNodeColor = oldUColor;
			//UPGRADE_TODO: Method 'javax.swing.JTabbedPane.setTitleAt' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'javax.swing.JTabbedPane.getSelectedIndex' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			canvas.Owner.tabbedPane.setTitleAt(canvas.Owner.tabbedPane.getSelectedIndex(), limid.Name);
			canvas.Refresh();
		}
		
		public virtual System.String ToString()
		{
			return "Edit network properties";
		}
	}
}