using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A table model used for creation of row headers for the node table editor.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	//UPGRADE_TODO: Interface 'javax.swing.table.TableModel' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
	public class ParentStateHeaders : TableModel
	{
		//UPGRADE_NOTE: Field 'EnclosingInstance' was added to class 'AnonymousClassTableModelListener' to access its enclosing instance. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1019"'
		private class AnonymousClassTableModelListener : TableModelListener
		{
			public AnonymousClassTableModelListener(ParentStateHeaders enclosingInstance)
			{
				InitBlock(enclosingInstance);
			}
			private void  InitBlock(ParentStateHeaders enclosingInstance)
			{
				this.enclosingInstance = enclosingInstance;
			}
			private ParentStateHeaders enclosingInstance;
			public ParentStateHeaders Enclosing_Instance
			{
				get
				{
					return enclosingInstance;
				}
				
			}
			//UPGRADE_TODO: Method 'javax.swing.table.TableModel.addTableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Class 'javax.swing.event.TableModelEvent' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			public virtual void  tableChanged(TableModelEvent e)
			{
				//UPGRADE_TODO: Method 'javax.swing.JTable.tableChanged' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				Enclosing_Instance.owner.tableChanged(e);
				Enclosing_Instance.owner.Refresh();
			}
		}
		virtual public int RowCount
		{
			get
			{
				return rowCount;
			}
			
		}
		virtual public int ColumnCount
		{
			get
			{
				return colCount;
			}
			
		}
		//UPGRADE_TODO: Class 'javax.swing.JTable' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		virtual public JTable Owner
		{
			set
			{
				owner = value;
			}
			
		}
		private Potential table;
		private int rowCount;
		private int colCount;
		private int parentNodeCount;
		private SupportClass.ListCollectionSupport listeners;
		private CommonProbabilityTableModel otherModel;
		//UPGRADE_TODO: Class 'javax.swing.JTable' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		private JTable owner;
		
		/// <summary> Creates a table model.</summary>
		/// <param name="node">The node of which the table(s) must be edited.
		/// </param>
		/// <param name="ptcm">The table model used for the potential table.
		/// </param>
		public ParentStateHeaders(Node node, CommonProbabilityTableModel ptcm)
		{
			table = (Potential) node.Table;
			if (node is TimeUtilityNode)
			{
				rowCount = table.Size;
				colCount = table.Nodes.Count;
				parentNodeCount = table.Nodes.Count;
			}
			else
			{
				rowCount = table.ParentStateCount;
				parentNodeCount = table.Nodes.Count - 1;
				colCount = parentNodeCount + 1;
			}
			listeners = new SupportClass.ListCollectionSupport();
			otherModel = ptcm;
			//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			otherModel.addTableModelListener(new AnonymousClassTableModelListener(this));
		}
		
		public ParentStateHeaders()
		{
			try
			{
				table = new DefaultProbabilityPotential(new NodeList());
			}
			catch (MLPException ex)
			{
			}
			rowCount = table.ParentStateCount;
			colCount = table.Nodes.Count - 1;
			parentNodeCount = table.Nodes.Count - 1;
			listeners = new SupportClass.ListCollectionSupport();
		}
		
		public virtual System.String getColumnName(int columnIndex)
		{
			if (columnIndex < parentNodeCount)
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				return table.Nodes[columnIndex].ToString();
			}
			else
				return "Sum";
		}
		
		public virtual System.Type getColumnClass(int columnIndex)
		{
			if (columnIndex < parentNodeCount)
				return "".GetType();
			else
				return (0).GetType();
		}
		
		public virtual bool isCellEditable(int rowIndex, int columnIndex)
		{
			return false;
		}
		
		public virtual System.Object getValueAt(int rowIndex, int columnIndex)
		{
			if (columnIndex < parentNodeCount)
			{
				int[] state = table.convertParentIndexToStateArray(rowIndex);
				return ((DiscreteNode) table.Nodes[columnIndex]).getStateLabel(state[columnIndex]);
			}
			else
				return (float) otherModel.getRowSum(rowIndex);
		}
		
		public virtual void  setValueAt(System.Object aValue, int rowIndex, int columnIndex)
		{
		}
		
		//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual void  addTableModelListener(TableModelListener l)
		{
			listeners.Add(l);
		}
		
		//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual void  removeTableModelListener(TableModelListener l)
		{
			listeners.RemoveElement(l);
		}
		
		public virtual void  noSum()
		{
			colCount = parentNodeCount;
		}
		
		public virtual void  includeSum()
		{
			colCount = parentNodeCount + 1;
		}
	}
}