using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A command pasting the contents of the (internal) clipboard.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// <p>Company: </p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class PasteCommand : Command
	{
		private LimidCanvas canvas;
		private SelectionSet selection;
		private NodeList copies;
		private SupportClass.ListCollectionSupport oldSelection;
		
		/// <summary> Creates a command object.</summary>
		/// <param name="sel">The content of the clipboard
		/// </param>
		/// <param name="can">The active canvas
		/// </param>
		public PasteCommand(SelectionSet sel, LimidCanvas can)
		{
			canvas = can;
			selection = sel;
		}
		
		public virtual void  execute()
		{
			try
			{
				LimidDomain activeLimid = canvas.Limid;
				copies = selection.copySelection();
				copies.moveToLimid(LimidFrame.nodeContainer, false);
				oldSelection = SupportClass.CreateCollection(canvas.SelectionSet);
				canvas.clearSelection();
				copies.moveToLimid(activeLimid, true);
				canvas.selectNodes(copies);
			}
			catch (MLPDuplicateNodeNameException ex)
			{
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.getMessage' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.ERROR_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, ex.Message, "Node name already exists in target Limid", JOptionPane.ERROR_MESSAGE);
			}
			//UPGRADE_NOTE: Exception 'java.lang.CloneNotSupportedException' was converted to 'System.Exception' which has different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1100"'
			catch (System.Exception ex)
			{
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.getMessage' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.ERROR_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, ex.Message, "Copying this kind of items not supported", JOptionPane.ERROR_MESSAGE);
			}
			catch (MLPOperationNotAllowedException ex)
			{
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.getMessage' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.ERROR_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, ex.Message, "The operation is not allowed", JOptionPane.ERROR_MESSAGE);
			}
		}
		
		public virtual void  undo()
		{
			System.Collections.IEnumerator ite = copies.GetEnumerator();
			try
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node node = (Node) ite.Current;
					canvas.Limid.removeNode(node);
				}
			}
			catch (MLPOperationNotAllowedException ex)
			{
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.getMessage' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.ERROR_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, ex.Message, "The operation is not allowed", JOptionPane.ERROR_MESSAGE);
			}
			canvas.clearSelection();
			canvas.SelectionSet.AddAll(oldSelection);
		}
		
		public virtual System.String ToString()
		{
			return "Paste";
		}
	}
}