using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A table model used for showing the contents of a probability table.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	
	public class ProbabilityTableContentModel : CommonProbabilityTableModel
	{
		virtual public int RowCount
		{
			get
			{
				return rowCount;
			}
			
		}
		virtual public int ColumnCount
		{
			get
			{
				return colCount;
			}
			
		}
		virtual public DefaultProbabilityPotential ProbabilityTable
		{
			get
			{
				return table;
			}
			
		}
		//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		virtual public System.Collections.ArrayList AllPotentials
		{
			get
			{
				//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
				System.Collections.ArrayList pot = new System.Collections.ArrayList();
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				pot.Add(table);
				return pot;
			}
			
		}
		private DefaultProbabilityPotential table;
		private int rowCount;
		private int colCount;
		private SupportClass.ListCollectionSupport listeners;
		
		/// <summary> Creates a table model.</summary>
		/// <param name="node">The node of which the probability table is to be edited.
		/// </param>
		public ProbabilityTableContentModel(DiscreteNode node)
		{
			// Make a copy of the table and edit the copy
			try
			{
				table = new DefaultProbabilityPotential(node.Table.Nodes);
				// Elegant, but inefficient, source code
				table.Data = node.Table.Data;
			}
			catch (MLPOperationNotAllowedException ex)
			{
				// This can never happen since a table for this node already exists
			}
			rowCount = table.ParentStateCount;
			colCount = table.StateCount;
			listeners = new SupportClass.ListCollectionSupport();
		}
		
		public ProbabilityTableContentModel()
		{
			try
			{
				table = new DefaultProbabilityPotential(new NodeList());
			}
			catch (MLPException ex)
			{
			}
			rowCount = table.ParentStateCount;
			colCount = table.StateCount;
			listeners = new SupportClass.ListCollectionSupport();
		}
		
		public virtual System.String getColumnName(int columnIndex)
		{
			return ((DiscreteNode) table.Nodes.GetLast()).getStateLabel(columnIndex);
		}
		
		public virtual System.Type getColumnClass(int columnIndex)
		{
			return (0).GetType();
		}
		
		public virtual bool isCellEditable(int rowIndex, int columnIndex)
		{
			return true;
		}
		
		public virtual System.Object getValueAt(int rowIndex, int columnIndex)
		{
			return new Float(table.getProbabilityProvider(rowIndex).get_Renamed(columnIndex));
		}
		
		public virtual void  setValueAt(System.Object aValue, int rowIndex, int columnIndex)
		{
			//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Float.floatValue' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			float value_Renamed = (float) ((System.Single) aValue);
			table.getProbabilityProvider(rowIndex).set_Renamed(columnIndex, value_Renamed);
			//UPGRADE_TODO: Class 'javax.swing.event.TableModelEvent' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Constructor 'javax.swing.event.TableModelEvent.TableModelEvent' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Field 'javax.swing.event.TableModelEvent.UPDATE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			TableModelEvent event_Renamed = new TableModelEvent(this, rowIndex, rowIndex, columnIndex, TableModelEvent.UPDATE);
			System.Collections.IEnumerator ite = listeners.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'javax.swing.event.TableModelListener.tableChanged' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				((TableModelListener) ite.Current).tableChanged(event_Renamed);
			}
		}
		
		//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual void  addTableModelListener(TableModelListener l)
		{
			listeners.Add(l);
		}
		
		//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual void  removeTableModelListener(TableModelListener l)
		{
			listeners.RemoveElement(l);
		}
		
		public virtual double getRowSum(int row)
		{
			double sum = 0;
			//UPGRADE_TODO: Method 'javax.swing.table.TableModel.getColumnCount' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			for (int i = 0; i < this.getColumnCount(); i++)
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Float.floatValue' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Method 'javax.swing.table.TableModel.getValueAt' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				sum = sum + (float) ((System.Single) this.getValueAt(row, i));
			}
			return sum;
		}
	}
}