using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A Table model used for displaying the probability table of a node. The class
	/// is currently not used by the API. The tables have been split in two: The
	/// ParentSTateHeaders displaying the row headers and the PobabilityTableContentModel
	/// displaying the probability values.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	
	//UPGRADE_TODO: Interface 'javax.swing.table.TableModel' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
	public class ProbabilityTableModel : TableModel
	{
		virtual public int RowCount
		{
			get
			{
				return rowCount;
			}
			
		}
		virtual public int ColumnCount
		{
			get
			{
				return colCount;
			}
			
		}
		private DefaultProbabilityPotential table;
		private int rowCount;
		private int colCount;
		private int parentNodeCount;
		private SupportClass.ListCollectionSupport listeners;
		
		public ProbabilityTableModel(DiscreteNode node)
		{
			table = (DefaultProbabilityPotential) node.Table;
			rowCount = table.ParentStateCount;
			colCount = table.Nodes.Count - 1 + table.StateCount;
			parentNodeCount = table.Nodes.Count - 1;
			listeners = new SupportClass.ListCollectionSupport();
		}
		
		public ProbabilityTableModel()
		{
			try
			{
				table = new DefaultProbabilityPotential(new NodeList());
			}
			catch (MLPException ex)
			{
			}
			rowCount = table.ParentStateCount;
			colCount = table.Nodes.Count - 1 + table.StateCount;
			parentNodeCount = table.Nodes.Count - 1;
			listeners = new SupportClass.ListCollectionSupport();
		}
		
		public virtual System.String getColumnName(int columnIndex)
		{
			if (columnIndex < parentNodeCount)
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				return table.Nodes[columnIndex].ToString();
			}
			return ((DiscreteNode) table.Nodes.GetLast()).getStateLabel(columnIndex - this.parentNodeCount);
		}
		
		public virtual System.Type getColumnClass(int columnIndex)
		{
			if (columnIndex < parentNodeCount)
				return "".GetType();
			return (0).GetType();
		}
		
		public virtual bool isCellEditable(int rowIndex, int columnIndex)
		{
			if (columnIndex < parentNodeCount)
				return false;
			return true;
		}
		
		public virtual System.Object getValueAt(int rowIndex, int columnIndex)
		{
			if (columnIndex < this.parentNodeCount)
			{
				int[] state = table.convertParentIndexToStateArray(rowIndex);
				return ((DiscreteNode) table.Nodes[columnIndex]).getStateLabel(state[columnIndex]);
			}
			else
				return new Float(table.getProbabilityProvider(rowIndex).get_Renamed(columnIndex - this.parentNodeCount));
		}
		
		public virtual void  setValueAt(System.Object aValue, int rowIndex, int columnIndex)
		{
			if (columnIndex >= this.parentNodeCount)
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Float.floatValue' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				float value_Renamed = (float) ((System.Single) aValue);
				table.getProbabilityProvider(rowIndex).set_Renamed(columnIndex - this.parentNodeCount, value_Renamed);
				//UPGRADE_TODO: Class 'javax.swing.event.TableModelEvent' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Constructor 'javax.swing.event.TableModelEvent.TableModelEvent' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Field 'javax.swing.event.TableModelEvent.UPDATE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				TableModelEvent event_Renamed = new TableModelEvent(this, rowIndex, rowIndex, columnIndex, TableModelEvent.UPDATE);
				System.Collections.IEnumerator ite = listeners.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'javax.swing.event.TableModelListener.tableChanged' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					((TableModelListener) ite.Current).tableChanged(event_Renamed);
				}
			}
		}
		
		//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual void  addTableModelListener(TableModelListener l)
		{
			listeners.Add(l);
		}
		
		//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual void  removeTableModelListener(TableModelListener l)
		{
			listeners.RemoveElement(l);
		}
	}
}