using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A command reordering the decision node order used by the Single Policy
	/// Updating algorithm.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// <p>Company: </p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	
	public class ReOrderingCommand : Command
	{
		private LimidCanvas canvas;
		private NodeList oldOrdering;
		private NodeList newOrdering;
		
		/// <summary> Creates a command object. The re-ordering is defined in a dialog with the user</summary>
		/// <param name="can">The active canvas.
		/// </param>
		public ReOrderingCommand(LimidCanvas can)
		{
			canvas = can;
			try
			{
				oldOrdering = new NodeList(canvas.Limid.DefaultOrdering);
				OrderingDialog dialog = new OrderingDialog(canvas.Owner, "Define decision node ordering", true, oldOrdering);
				//UPGRADE_TODO: Method 'java.awt.Component.setVisible' was converted to 'System.Windows.Forms.Control.Visible' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073"'
				//UPGRADE_TODO: System.Windows.Forms.Application.Run must be called to start a main form. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1135"'
				dialog.Visible = true;
				newOrdering = dialog.NewOrdering;
			}
			catch (MLPNetworkNotCompiledException ex)
			{
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.getMessage' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.ERROR_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, ex.Message, "Not compiled error", JOptionPane.ERROR_MESSAGE);
			}
		}
		
		public virtual void  execute()
		{
			try
			{
				canvas.Limid.DefaultOrdering = newOrdering;
			}
			catch (MLPNetworkNotCompiledException ex)
			{
				// If we could do it the first time, then it IS compiled ...
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.WARNING_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, "An unexpected error " + "was detected. Please inform the " + "developer. The exception thrown was:\n" + ex.ToString() + "\nYou should save your work!", "Software BUG?", JOptionPane.WARNING_MESSAGE);
			}
			catch (MLPInvalidNodeListException ex)
			{
				// Will never happen ...
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.WARNING_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, "An unexpected error " + "was detected. Please inform the " + "developer. The exception thrown was:\n" + ex.ToString() + "\nYou should save your work!", "Software BUG?", JOptionPane.WARNING_MESSAGE);
			}
		}
		public virtual void  undo()
		{
			try
			{
				canvas.Limid.DefaultOrdering = oldOrdering;
			}
			catch (MLPNetworkNotCompiledException ex)
			{
				// If we could do it the first time, then it IS compiled ...
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.WARNING_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, "An unexpected error " + "was detected. Please inform the " + "developer. The exception thrown was:\n" + ex.ToString() + "\nYou should save your work!", "Software BUG?", JOptionPane.WARNING_MESSAGE);
			}
			catch (MLPInvalidNodeListException ex)
			{
				// Will never happen ...
				//UPGRADE_TODO: Method 'javax.swing.JOptionPane.showMessageDialog' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Field 'javax.swing.JOptionPane.WARNING_MESSAGE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				JOptionPane.showMessageDialog(canvas.Owner, "An unexpected error " + "was detected. Please inform the " + "developer. The exception thrown was:\n" + ex.ToString() + "\nYou should save your work!", "Software BUG?", JOptionPane.WARNING_MESSAGE);
			}
		}
		
		public virtual System.String ToString()
		{
			return "Re-ordering of decisions";
		}
	}
}