using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A command moving the nodes and rescaling the distances of a Limid.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// <p>Company: </p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class RelocateCommand : Command
	{
		internal LimidCanvas canvas;
		internal LimidDomain limid;
		internal System.Drawing.Size newDimension;
		internal double newXOffset;
		internal double newYOffset;
		internal double xScale;
		internal double yScale;
		
		/// <summary> Creates a command object.</summary>
		/// <param name="can">The active canvas
		/// </param>
		/// <param name="dim">The desired new dimension of the Limid
		/// </param>
		public RelocateCommand(LimidCanvas can, System.Drawing.Size dim)
		{
			newDimension = dim;
			canvas = can;
			limid = canvas.Limid;
			double[] extremes = limid.ExtremePositions;
			double minX = extremes[0];
			double maxX = extremes[1];
			double minY = extremes[2];
			double maxY = extremes[3];
			double xDif = maxX - minX;
			double yDif = maxY - minY;
			newXOffset = minX;
			newYOffset = minY;
			//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			xScale = (dim.getWidth() - limid.getNodeSize().getWidth()) / xDif;
			//UPGRADE_TODO: Method 'java.awt.Dimension.getHeight' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			yScale = (dim.getHeight() - limid.getNodeSize().getHeight()) / yDif;
		}
		
		public virtual void  execute()
		{
			limid.reLocateToFitDimension(newXOffset, xScale, newYOffset, yScale, true);
			canvas.Refresh();
		}
		
		public virtual void  undo()
		{
			limid.reLocateToFitDimension(- newXOffset, (1 / xScale), - newYOffset, (1 / yScale), false);
			canvas.Refresh();
		}
		
		public virtual System.String ToString()
		{
			return "Relocate nodes";
		}
	}
}