using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> An implementation of a Collection used for holding selected items.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
	public class SelectionSet : SupportClass.CollectionSupport
	{
		public virtual int Count
		{
			get
			{
				return selected.Count;
			}
			
		}
		/// <summary> Returns the items that are nodes.</summary>
		/// <returns> A list of nodes.
		/// </returns>
		virtual public NodeList SelectedNodes
		{
			get
			{
				NodeList selectedNodes = new NodeList();
				System.Collections.IEnumerator ite = this.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Drawable shape = (Drawable) ite.Current;
					if (shape is Node)
						selectedNodes.Add(shape);
				}
				return selectedNodes;
			}
			
		}
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		private SupportClass.CollectionSupport selected;
		//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		private System.Collections.ArrayList listeners;
		
		/// <summary> Creates a selection set with an initial collection of selected items</summary>
		/// <param name="c">The initially selected items
		/// </param>
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		public SelectionSet(SupportClass.CollectionSupport c)
		{
			selected = c;
			//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			listeners = new System.Collections.ArrayList();
		}
		
		public virtual bool IsEmpty()
		{
			return selected.IsEmpty();
		}
		
		public virtual bool Contains(System.Object o)
		{
			return selected.Contains(o);
		}
		
		public virtual System.Collections.IEnumerator GetEnumerator()
		{
			return selected.GetEnumerator();
		}
		
		public virtual System.Object[] ToArray()
		{
			return selected.ToArray();
		}
		
		public virtual System.Object[] ToArray(System.Object[] a)
		{
			return selected.ToArray(a);
		}
		
		public virtual bool Add(System.Object o)
		{
			bool toReturn = selected.Add(o);
			if (toReturn && selected.Count == 1)
				this.fireSelectionCreated();
			return toReturn;
		}
		
		public virtual bool Remove(System.Object o)
		{
			bool toReturn = selected.Remove(o);
			if (toReturn && selected.Count == 0)
				this.fireSelectionCleared();
			return toReturn;
		}
		
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		public virtual bool ContainsAll(SupportClass.CollectionSupport c)
		{
			return selected.ContainsAll(c);
		}
		
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		public virtual bool AddAll(SupportClass.CollectionSupport c)
		{
			bool wasEmpty = selected.IsEmpty();
			bool toReturn = selected.AddAll(c);
			if (toReturn && wasEmpty && selected.Count > 0)
				this.fireSelectionCreated();
			return toReturn;
		}
		
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		public virtual bool RemoveAll(SupportClass.CollectionSupport c)
		{
			bool hadElements = (!IsEmpty());
			bool toReturn = selected.RemoveAll(c);
			if (toReturn && hadElements && selected.Count == 0)
				this.fireSelectionCleared();
			return toReturn;
		}
		
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		public virtual bool RetainAll(SupportClass.CollectionSupport c)
		{
			bool hadElements = (!IsEmpty());
			bool toReturn = selected.RetainAll(c);
			if (toReturn && hadElements && selected.Count == 0)
				this.fireSelectionCleared();
			return toReturn;
		}
		
		public virtual void  Clear()
		{
			bool hadElements = (!IsEmpty());
			selected.Clear();
			if (hadElements)
				this.fireSelectionCleared();
		}
		
		public  virtual bool Equals(System.Object o)
		{
			return SupportClass.EqualsSupport(selected, o);
		}
		
		public virtual void  addLimidSelectionListener(LimidSelectionListener aListener)
		{
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			this.listeners.Add(aListener);
		}
		
		public virtual bool removeLimidSelectionListener(LimidSelectionListener aListener)
		{
			return listeners.Remove(aListener);
		}
		
		protected internal virtual void  fireSelectionCleared()
		{
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			for (int i = 0; i < listeners.Count; i++)
				((LimidSelectionListener) listeners[i]).selectionCleared();
		}
		
		protected internal virtual void  fireSelectionCreated()
		{
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			for (int i = 0; i < listeners.Count; i++)
				((LimidSelectionListener) listeners[i]).selectionCreated();
		}
		
		/// <summary> Returns a list of copies (clones) of the selected nodes</summary>
		/// <returns> A list of copies.
		/// @throws MLPOperationNotAllowedException
		/// @throws CloneNotSupportedException
		/// </returns>
		public virtual NodeList copySelection()
		{
			NodeList nodes = this.SelectedNodes;
			NodeList copies = nodes.copy();
			// Remove all edges
			System.Collections.IEnumerator ite = copies.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				node.removeAllParents();
				node.removeAllChildren();
			}
			// Add selected edges
			// First, establish a separate list of selected edges
			SupportClass.ListCollectionSupport edgeList = SupportClass.CreateCollection(this);
			// Remove the nodes
			edgeList.RemoveAll(nodes);
			// Now, the list only contains edges
			ite = edgeList.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				DirectedEdge edge = (DirectedEdge) ite.Current;
				Node owner = edge.Owner;
				Node parent = edge.Neighbor;
				int ownerIndex = nodes.IndexOf(owner);
				int parentIndex = nodes.IndexOf(parent);
				// The order of the nodes is the same in the original list and the clone
				Node ownerClone;
				if (ownerIndex != - 1)
					ownerClone = (Node) copies[ownerIndex];
				else
					ownerClone = owner;
				Node parentClone;
				// If the parent is among the selected nodes
				if (parentIndex != - 1)
					parentClone = (Node) copies[parentIndex];
				// Otherwise link to external node
				else
					parentClone = parent;
				ownerClone.addParent(parentClone);
			}
			ite = copies.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				node.reDoEdges();
			}
			return copies;
		}
	}
}