using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A table model used for displaying the state labels of a node in a table.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	//UPGRADE_TODO: Interface 'javax.swing.table.TableModel' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
	public class StateLabelTableModel : TableModel
	{
		virtual public int RowCount
		{
			get
			{
				return node.getNumberOfStates();
			}
			
		}
		virtual public int ColumnCount
		{
			get
			{
				if (node.hasStateValues())
					return 2;
				return 1;
			}
			
		}
		private DiscreteNode node;
		private StateValues valueNode = null;
		private SupportClass.ListCollectionSupport listeners;
		
		/// <summary> Creates a table model.</summary>
		/// <param name="node">The source node.
		/// </param>
		public StateLabelTableModel(DiscreteNode node)
		{
			this.node = node;
			if (node is StateValues)
				valueNode = (StateValues) node;
			listeners = new SupportClass.ListCollectionSupport();
		}
		
		public virtual System.String getColumnName(int columnIndex)
		{
			if (columnIndex == 0)
				return "State label";
			else
				return "State value";
		}
		
		public virtual System.Type getColumnClass(int columnIndex)
		{
			if (columnIndex == 0)
				return "".GetType();
			else
				return (0).GetType();
		}
		
		public virtual bool isCellEditable(int rowIndex, int columnIndex)
		{
			return true;
		}
		
		public virtual System.Object getValueAt(int rowIndex, int columnIndex)
		{
			if (columnIndex == 0)
				return node.getStateLabel(rowIndex);
			else
			{
				if (valueNode != null)
					return valueNode.getStateValue(rowIndex);
				else
					return "Not available";
			}
		}
		
		public virtual void  setValueAt(System.Object aValue, int rowIndex, int columnIndex)
		{
			if (columnIndex == 0)
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				System.String label = aValue.ToString();
				node.setStateLabel(rowIndex, label);
			}
			else
			{
				if (valueNode != null)
				{
					double value_Renamed = ((System.Double) aValue);
					valueNode.setStateValue(rowIndex, value_Renamed);
				}
			}
			//UPGRADE_TODO: Class 'javax.swing.event.TableModelEvent' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Constructor 'javax.swing.event.TableModelEvent.TableModelEvent' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Field 'javax.swing.event.TableModelEvent.UPDATE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			TableModelEvent event_Renamed = new TableModelEvent(this, rowIndex, rowIndex, columnIndex, TableModelEvent.UPDATE);
			System.Collections.IEnumerator ite = listeners.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'javax.swing.event.TableModelListener.tableChanged' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				((TableModelListener) ite.Current).tableChanged(event_Renamed);
			}
		}
		
		//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual void  addTableModelListener(TableModelListener l)
		{
			listeners.Add(l);
		}
		
		//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual void  removeTableModelListener(TableModelListener l)
		{
			listeners.RemoveElement(l);
		}
	}
}