using System;
using dk.esthauge.limid.api;
using dk.kvl.ihh.prodstyr.mlhmp;
namespace dk.esthauge.limid.gui
{
	
	/// <summary> A table model used for displaying the contents of a utility table or an other
	/// table of a utility node.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class UtilityTableModel : CommonProbabilityTableModel
	{
		virtual public int RowCount
		{
			get
			{
				return this.rowCount;
			}
			
		}
		virtual public int ColumnCount
		{
			get
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				return potentials.Count;
			}
			
		}
		virtual public DefaultProbabilityPotential ProbabilityTable
		{
			get
			{
				try
				{
					DefaultProbabilityPotential pot = new DefaultProbabilityPotential(node.Table.Nodes);
					pot.Data = ((PotentialTable) potentials[0]).Data;
					return pot;
				}
				catch (MLPOperationNotAllowedException ex)
				{
					// Never happens
				}
				return null;
			}
			
		}
		//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		virtual public System.Collections.ArrayList AllPotentials
		{
			get
			{
				return this.potentials;
			}
			
		}
		private TimeUtilityNode node;
		//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		private System.Collections.ArrayList potentials;
		private SupportClass.ListCollectionSupport listeners;
		private int rowCount;
		
		/// <summary> Creates a table model.</summary>
		/// <param name="node">The utility node.
		/// </param>
		public UtilityTableModel(TimeUtilityNode node)
		{
			this.node = node;
			//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			System.Collections.ArrayList tables = node.AllPotentials;
			//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			this.potentials = new System.Collections.ArrayList();
			listeners = new SupportClass.ListCollectionSupport();
			// Make a copy of the tables and edit the copies
			try
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				for (int i = 0; i < tables.Count; i++)
				{
					PotentialTable table = new PotentialTable(node.Table.Nodes);
					double[] data1 = ((Potential) tables[i]).Data;
					double[] data2 = new double[data1.Length];
					for (int j = 0; j < data1.Length; j++)
						data2[j] = data1[j];
					table.Data = data2;
					//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					potentials.Add(table);
				}
			}
			catch (MLPOperationNotAllowedException ex)
			{
				// This can never happen since a table for this node already exists
			}
			rowCount = node.Table.Size;
		}
		
		public virtual System.String getColumnName(int columnIndex)
		{
			if (columnIndex == 0)
				return "Utility";
			if (columnIndex == 1)
				return "Time";
			return "Item " + columnIndex;
		}
		
		public virtual System.Type getColumnClass(int columnIndex)
		{
			return (0).GetType();
		}
		
		public virtual bool isCellEditable(int rowIndex, int columnIndex)
		{
			return true;
		}
		
		public virtual System.Object getValueAt(int rowIndex, int columnIndex)
		{
			return ((Potential) potentials[columnIndex]).getDataItem(rowIndex);
		}
		
		public virtual void  setValueAt(System.Object aValue, int rowIndex, int columnIndex)
		{
			double value_Renamed = ((System.Double) aValue);
			((Potential) potentials[columnIndex]).setDataItem(rowIndex, value_Renamed);
			//UPGRADE_TODO: Class 'javax.swing.event.TableModelEvent' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Constructor 'javax.swing.event.TableModelEvent.TableModelEvent' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Field 'javax.swing.event.TableModelEvent.UPDATE' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			TableModelEvent event_Renamed = new TableModelEvent(this, rowIndex, rowIndex, columnIndex, TableModelEvent.UPDATE);
			System.Collections.IEnumerator ite = listeners.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'javax.swing.event.TableModelListener.tableChanged' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				((TableModelListener) ite.Current).tableChanged(event_Renamed);
			}
		}
		
		//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual void  addTableModelListener(TableModelListener l)
		{
			listeners.Add(l);
		}
		
		//UPGRADE_TODO: Interface 'javax.swing.event.TableModelListener' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual void  removeTableModelListener(TableModelListener l)
		{
			listeners.RemoveElement(l);
		}
		
		public virtual double getRowSum(int row)
		{
			// It is not clear what this can be used for
			double sum = 0;
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			for (int i = 0; i < potentials.Count; i++)
				sum = sum + ((Potential) potentials[i]).getDataItem(row);
			return sum;
		}
	}
}