using System;
namespace dk.esthauge.limid.parser
{
	
	/// <summary> A parser tree node representing a list of floating point numericals.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class FloatListNode:SimpleNode
	{
		/// <summary> Returns the list of float values</summary>
		/// <returns> An array of float
		/// </returns>
		virtual public float[] Floats
		{
			get
			{
				float[] fs = new float[floatList.Count];
				System.Double[] fos = new System.Double[floatList.Count];
				floatList.ToArray(fos);
				for (int i = 0; i < fs.Length; i++)
					fs[i] = (float) fos[i];
				return fs;
			}
			
		}
		private SupportClass.ListCollectionSupport floatList;
		
		/// <summary> Creates a parser tree node.</summary>
		/// <param name="t">A token representing a floating point value
		/// </param>
		public FloatListNode(Token t):base(- 3)
		{
			// Take the token given as a parameter
			Token current = t;
			// Create a linked list
			floatList = new SupportClass.ListCollectionSupport();
			// Remember the "kind" of node (a FLOAT)
			int kind = t.kind;
			// We know that it is a float, so convert it
			//UPGRADE_TODO: Format of parameters of constructor 'java.lang.Double.Double' are different in the equivalent in .NET. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1092"'
			System.Double f = System.Double.Parse(current.image);
			// Add it to the list as the first element
			floatList.Add(f);
			// Add converted tokens as long as they are of kind FLOAT
			while (current.next != null && current.next.kind == kind)
			{
				current = current.next;
				//UPGRADE_TODO: Format of parameters of constructor 'java.lang.Double.Double' are different in the equivalent in .NET. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1092"'
				f = System.Double.Parse(current.image);
				floatList.Add(f);
			}
		}
		
		/// <summary> A descriptive name</summary>
		/// <returns> A description
		/// </returns>
		public override System.String ToString()
		{
			return "FloatList";
		}
	}
}