using System;
namespace dk.esthauge.limid.parser
{
	
	/// <summary> A parser tree node representing a list of integers.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class IntListNode:SimpleNode
	{
		/// <summary> Returns the list of integers.</summary>
		/// <returns> An array of int.
		/// </returns>
		virtual public int[] Integers
		{
			get
			{
				int[] fs = new int[intList.Count];
				System.Int32[] fos = new System.Int32[intList.Count];
				intList.ToArray(fos);
				for (int i = 0; i < fs.Length; i++)
					fs[i] = fos[i];
				return fs;
			}
			
		}
		private SupportClass.ListCollectionSupport intList;
		
		/// <summary> Creates a parser tree node.</summary>
		/// <param name="t">A token representing an integer point value
		/// </param>
		public IntListNode(Token t):base(- 2)
		{
			// Take the token given as a parameter
			Token current = t;
			// Create a linked list
			intList = new SupportClass.ListCollectionSupport();
			// Remember the "kind" of node (a NONNEGINT)
			int kind = t.kind;
			// We know that it is an integer, so convert it
			System.Int32 f = System.Int32.Parse(current.image);
			// Add it to the list as the first element
			intList.Add(f);
			// Add converted tokens as long as they are of kind NONNEGINT
			while (current.next != null && current.next.kind == kind)
			{
				current = current.next;
				f = System.Int32.Parse(current.image);
				intList.Add(f);
			}
		}
		
		/// <summary> Returns a descriptive name</summary>
		/// <returns> A description
		/// </returns>
		public override System.String ToString()
		{
			return "IntegerList";
		}
	}
}