using System;
namespace dk.esthauge.limid.parser
{
	
	/// <summary> A parse tree node representing an attribute name and the corresponding value.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class NodeAttribute:SimpleNode
	{
		/// <summary> Returns the name of the attribute.</summary>
		/// <returns> The name.
		/// </returns>
		virtual public System.String Name
		{
			get
			{
				return name;
			}
			
		}
		/// <summary> Returns the value of the attribute as an integer. May throw a runtime
		/// exception.
		/// </summary>
		/// <returns> The value.
		/// </returns>
		virtual public int Value
		{
			get
			{
				return (System.Int32.Parse(value_Renamed));
			}
			
		}
		/// <summary> Returns the value of the attribute as a string.</summary>
		/// <returns> The value.
		/// </returns>
		virtual public System.String StringValue
		{
			get
			{
				return value_Renamed;
			}
			
		}
		/// <summary> Returns the value of the attribute as a float. May throw a runtime
		/// exception.
		/// </summary>
		/// <returns> The value.
		/// </returns>
		virtual public float FloatValue
		{
			get
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Float.floatValue' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Format of parameters of constructor 'java.lang.Float.Float' are different in the equivalent in .NET. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1092"'
				return (float) (System.Single.Parse(value_Renamed));
			}
			
		}
		/// <summary> Returns the value of the attribute as a double. May throw a runtime
		/// exception.
		/// </summary>
		/// <returns> The value.
		/// </returns>
		virtual public double DoubleValue
		{
			get
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Float.floatValue' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_TODO: Format of parameters of constructor 'java.lang.Float.Float' are different in the equivalent in .NET. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1092"'
				return (float) (System.Single.Parse(value_Renamed));
			}
			
		}
		private System.String name, value_Renamed;
		
		/// <summary> Creates a parse tree node</summary>
		/// <param name="n">The name of the attribute.
		/// </param>
		/// <param name="v">The value of the attribute (as a string)
		/// </param>
		public NodeAttribute(System.String n, System.String v):base(- 1)
		{
			name = n;
			if ((System.Object) v == null)
			{
				value_Renamed = "#IMPLIED";
			}
			else
			{
				value_Renamed = v;
			}
		}
		
		/// <summary> Returns the string "attribute: <i>name</i> = <I>value</i>" where <i>name</i>
		/// and <i>value</i> are replaced by the properties of the attribute in question.
		/// </summary>
		/// <returns> A string representation of the attribute
		/// </returns>
		public override System.String ToString()
		{
			return "attribute: " + name + "=" + value_Renamed;
		}
	}
}