using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.test
{
	
	/// <summary> <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// <p>Company: </p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	
	public class CompilationTester
	{
		internal LimidDomain limid;
		
		public CompilationTester(LimidDomain ld)
		{
			limid = ld;
		}
		
		public CompilationTester(System.String filename)
		{
			try
			{
				ModelReader reader = new ModelReader(filename);
				limid = reader.readLimid();
			}
			catch (System.Exception ex)
			{
				SupportClass.WriteStackTrace(ex, Console.Error);
			}
		}
		
		public virtual bool testMoralization()
		{
			bool testResult = true;
			limid.moralize();
			NodeList nodes = limid.Nodes;
			System.Collections.IEnumerator ite = nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				NodeList children = node.Children;
				System.Collections.IEnumerator jte = nodes.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (jte.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node otherNode = (Node) jte.Current;
					if (node != otherNode)
					{
						NodeList otherChildren = otherNode.Children;
						NodeList commonChildren = children.getIntersectionWith(otherChildren);
						if (commonChildren.Count > 0)
						{
							if (!node.UndirectedEdges.Contains(otherNode))
								return false;
						}
					}
				}
			}
			return testResult;
		}
		
		public virtual bool testTriangulation()
		{
			limid.deCompile();
			limid.compile();
			bool testResult = true;
			int i = 1;
			NodeList V = limid.Nodes;
			V.RemoveAll(limid.UtilityNodes);
			int[] c = new int[V.Count];
			System.Collections.IEnumerator ite = V.GetEnumerator();
			int nodeIndex = 0;
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				c[nodeIndex] = 0;
				nodeIndex++;
			}
			NodeList L = new NodeList();
			while (L.Count != V.Count)
			{
				NodeList U = new NodeList(V);
				U.RemoveAll(L);
				ite = U.GetEnumerator();
				int max = - 1;
				Node maxC = null;
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node node = (Node) ite.Current;
					int index = V.IndexOf(node);
					if (c[index] > max)
					{
						maxC = node;
						max = c[index];
					}
				}
				NodeList maxCNeighbors = new NodeList(maxC.UndirectedEdges);
				NodeList PI = new NodeList();
				ite = maxCNeighbors.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node node = (Node) ite.Current;
					if (L.Contains(node))
						PI.Add(node);
				}
				ite = PI.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node node = (Node) ite.Current;
					NodeList family = new NodeList(node.UndirectedEdges);
					family.Add(node);
					if (!family.ContainsAll(PI))
						return false;
				}
				// If still alive: PI is complete!
				NodeList W = new NodeList();
				ite = maxCNeighbors.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node node = (Node) ite.Current;
					if (U.Contains(node))
						W.Add(node);
				}
				ite = W.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node node = (Node) ite.Current;
					int index = V.IndexOf(node);
					c[index] = c[index] + 1;
				}
				L.Add(maxC);
				i++;
			}
			return testResult;
		}
		
		public virtual bool testCliques()
		{
			bool testResult = true;
			LimidJunctionTree jTree = limid.JunctionTree;
			SupportClass.ListCollectionSupport cliques = jTree.Cliques;
			// Iterate over cliques
			System.Collections.IEnumerator ite = cliques.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				LimidClique clique = (LimidClique) ite.Current;
				// Create a list of external neighbor nodes
				NodeList neighborNodes = new NodeList();
				// Check that the sub graph is complete
				NodeList members = clique.Members;
				System.Collections.IEnumerator jte = members.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (jte.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node node = (Node) jte.Current;
					// Find all neighbors
					NodeList family = new NodeList(node.UndirectedEdges);
					// Find the neighbors NOT belonging to this clique
					System.Collections.IEnumerator kte = family.GetEnumerator();
					//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
					while (kte.MoveNext())
					{
						//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
						System.Object neighbor = kte.Current;
						if (!members.Contains(neighbor))
							neighborNodes.Add(neighbor);
					}
					// Create the family of the present node
					family.Add(node);
					// If the family does not include the whole clique, the sub graph
					// is not complete
					if (!family.ContainsAll(members))
						return false;
				}
				// Check that the complete sub graph is maximal, i.e. that no neighbor
				// can be added without making the graph incomplete
				jte = neighborNodes.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (jte.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node node = (Node) jte.Current;
					// Find all neighbors
					NodeList neighbors = new NodeList(node.UndirectedEdges);
					// If the neighbors includes the whole clique, then the complete sub
					// graph is not maximal
					if (neighbors.ContainsAll(members))
						return false;
				}
			}
			return testResult;
		}
		
		public virtual bool testJunctionTree()
		{
			bool testResult = true;
			LimidJunctionTree jTree = limid.JunctionTree;
			SupportClass.ListCollectionSupport cliques = jTree.Cliques;
			System.Collections.IEnumerator ite = cliques.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				LimidClique clique = (LimidClique) ite.Current;
				System.Collections.IEnumerator jte = cliques.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (jte.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					LimidClique otherClique = (LimidClique) jte.Current;
					if (clique != otherClique)
					{
						NodeList intersection = clique.Members.getIntersectionWith(otherClique.Members);
						if (intersection.Count > 0)
						{
							SupportClass.ListCollectionSupport path = clique.getPathToClique(null, otherClique);
							System.Collections.IEnumerator kte = path.GetEnumerator();
							//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
							while (kte.MoveNext())
							{
								//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
								System.Object cliqueOrSeparator = kte.Current;
								// Is it a clique?
								if (cliqueOrSeparator is LimidClique)
								{
									// Yes, it is a clique
									LimidClique cliqueOnPath = (LimidClique) cliqueOrSeparator;
									if (!cliqueOnPath.Members.ContainsAll(intersection))
										return false;
								}
								else
								{
									// It is NOT a clique. Then it must be a separator
									LimidSeparator separator = (LimidSeparator) cliqueOrSeparator;
									if (!separator.SeparatorMembers.ContainsAll(intersection))
										return false;
									// Check the references
									LimidClique left = separator.getLeftClique();
									LimidClique right = separator.getRightClique();
									if (!cliques.Contains(left))
										return false;
									if (!cliques.Contains(right))
										return false;
									if (!left.Separators.Contains(separator))
										return false;
									if (!right.Separators.Contains(separator))
										return false;
									if (!left.Neighbors.Contains(right))
										return false;
									if (!right.Neighbors.Contains(left))
										return false;
								}
							}
						}
					}
				}
			}
			return testResult;
		}
		
		public virtual void  testCompilation()
		{
			try
			{
				System.String name = limid.Name;
				System.Console.Out.WriteLine();
				System.Console.Out.WriteLine("Limid " + name + ", " + limid.Nodes.Count + " nodes:");
				System.Console.Out.WriteLine(name + " is moralized: " + testMoralization());
				System.Console.Out.WriteLine(name + " is triangulated: " + testTriangulation());
				System.Console.Out.WriteLine(name + " has valid cliques: " + testCliques());
				System.Console.Out.WriteLine(name + " has valid junction tree: " + testJunctionTree());
			}
			catch (System.Exception ex)
			{
				SupportClass.WriteStackTrace(ex, Console.Error);
			}
		}
	}
}