using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.test
{
	
	/// <summary> <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	
	public class ExceptionTester
	{
		
		public static bool testParentExceptions()
		{
			bool testResult = true;
			LimidDomain limid1 = new LimidDomain("Test domain 1");
			LimidDomain limid2 = new LimidDomain("Test domain 2");
			try
			{
				LimidChanceNode c1_1 = new LimidChanceNode(limid1);
				LimidChanceNode c2_1 = new LimidChanceNode(limid1);
				LimidChanceNode c3_1 = new LimidChanceNode(limid1);
				LimidChanceNode c4_1 = new LimidChanceNode(limid1);
				DiscreteDecisionChanceNode d1_1 = new DiscreteDecisionChanceNode(limid1);
				DiscreteDecisionChanceNode d2_1 = new DiscreteDecisionChanceNode(limid1);
				DiscreteDecisionChanceNode d3_1 = new DiscreteDecisionChanceNode(limid1);
				DiscreteDecisionChanceNode d4_1 = new DiscreteDecisionChanceNode(limid1);
				TimeUtilityNode u1_1 = new TimeUtilityNode(limid1);
				TimeUtilityNode u2_1 = new TimeUtilityNode(limid1);
				TimeUtilityNode u3_1 = new TimeUtilityNode(limid1);
				TimeUtilityNode u4_1 = new TimeUtilityNode(limid1);
				
				LimidChanceNode c1_2 = new LimidChanceNode(limid2);
				LimidChanceNode c2_2 = new LimidChanceNode(limid2);
				LimidChanceNode c3_2 = new LimidChanceNode(limid2);
				LimidChanceNode c4_2 = new LimidChanceNode(limid2);
				DiscreteDecisionChanceNode d1_2 = new DiscreteDecisionChanceNode(limid2);
				DiscreteDecisionChanceNode d2_2 = new DiscreteDecisionChanceNode(limid2);
				DiscreteDecisionChanceNode d3_2 = new DiscreteDecisionChanceNode(limid2);
				DiscreteDecisionChanceNode d4_2 = new DiscreteDecisionChanceNode(limid2);
				TimeUtilityNode u1_2 = new TimeUtilityNode(limid2);
				TimeUtilityNode u2_2 = new TimeUtilityNode(limid2);
				TimeUtilityNode u3_2 = new TimeUtilityNode(limid2);
				TimeUtilityNode u4_2 = new TimeUtilityNode(limid2);
				
				// Test for parent and child identical
				try
				{
					c1_1.addParent(c1_1);
					return false;
				}
				catch (MLPParentAndChildIdenticalException ex)
				{
					System.Console.Out.WriteLine("MLPParentAndChildIdenticalException thrown correctly");
				}
				
				// Test for same parent added twice
				try
				{
					c1_1.addParent(c2_1);
					c1_1.addParent(c2_1);
					return false;
				}
				catch (MLPParentAlreadyDefinedException ex)
				{
					c1_1.removeAllParents();
					System.Console.Out.WriteLine("MLPParentAlreadyDefinedException thrown correctly");
				}
				
				// Test for utility parent
				try
				{
					c1_1.addParent(u1_1);
					return false;
				}
				catch (MLPUtilityParentNotAllowedException ex)
				{
					System.Console.Out.WriteLine("MLPUtilityParentNotAllowedException thrown correctly");
				}
				
				// Test for cycle
				try
				{
					c1_1.addParent(d1_1);
					d1_1.addParent(c2_1);
					c2_1.addParent(d2_1);
					d2_1.addParent(c3_1);
					c3_1.addParent(d3_1);
					d3_1.addParent(c4_1);
					c4_1.addParent(d4_1);
					// The next line produces a cycle
					d4_1.addParent(c1_1);
					return false;
				}
				catch (MLPCyclicNetException ex)
				{
					// Clean up
					c1_1.removeAllParents();
					d1_1.removeAllParents();
					c2_1.removeAllParents();
					d2_1.removeAllParents();
					c3_1.removeAllParents();
					d3_1.removeAllParents();
					c4_1.removeAllParents();
					d4_1.removeAllParents();
					System.Console.Out.WriteLine("MLPCyclicNetException thrown correctly");
				}
				
				// Test for cycle
				try
				{
					c1_1.addParent(d1_1);
					// The next line produces a cycle
					d1_1.addParent(c1_1);
					return false;
				}
				catch (MLPCyclicNetException ex)
				{
					c1_1.removeAllParents();
					System.Console.Out.WriteLine("MLPCyclicNetException thrown correctly");
				}
				
				// Test for different LIMID
				try
				{
					c1_1.addParent(c1_2);
					return false;
				}
				catch (MLPNodeNotMemberException ex)
				{
					System.Console.Out.WriteLine("MLPNodeNotMemberException thrown correctly");
				}
				
				// Test for name conflict
				try
				{
					c1_1.setName(u4_1.getName());
					return false;
				}
				catch (MLPDuplicateNodeNameException ex)
				{
					System.Console.Out.WriteLine("MLPDuplicateNodeNameException thrown correctly");
				}
				
				// Test for not not existing names
				try
				{
					limid1.getNodeByName("Viggo");
					return false;
				}
				catch (MLPNodeNotFoundException ex)
				{
					System.Console.Out.WriteLine("MLPNodeNotFoundException thrown correctly");
				}
				
				// Test for connectivity
				try
				{
					limid1.compile();
					return false;
				}
				catch (MLPGraphNotConnectedException ex)
				{
					System.Console.Out.WriteLine("MLPGraphNotConnectedException thrown correctly");
				}
				
				// Add edges to create a 4 stage Markov decision process
				d1_1.addParent(c1_1);
				u1_1.addParent(c1_1);
				u1_1.addParent(d1_1);
				
				c2_1.addParent(c1_1);
				c2_1.addParent(d1_1);
				d2_1.addParent(c2_1);
				u2_1.addParent(c2_1);
				u2_1.addParent(d2_1);
				
				c3_1.addParent(c2_1);
				c3_1.addParent(d2_1);
				d3_1.addParent(c3_1);
				u3_1.addParent(c3_1);
				u3_1.addParent(d3_1);
				
				c4_1.addParent(c3_1);
				c4_1.addParent(d3_1);
				d4_1.addParent(c4_1);
				u4_1.addParent(c4_1);
				u4_1.addParent(d4_1);
				
				// Provoke a not compiled exception
				NodeList myOrdering = new NodeList();
				myOrdering.Add(d1_1);
				myOrdering.Add(d4_1);
				myOrdering.Add(d3_1);
				myOrdering.Add(d2_1);
				try
				{
					limid1.DefaultOrdering = myOrdering;
					return false;
				}
				catch (MLPNetworkNotCompiledException ex)
				{
					System.Console.Out.WriteLine("MLPNetworkNotCompiledException thrown correctly " + "when setting decision node ordering");
				}
				// Provoke a not compiled exception
				try
				{
					limid1.singlePolicyUpdating();
					return false;
				}
				catch (MLPNetworkNotCompiledException ex)
				{
					System.Console.Out.WriteLine("MLPNetworkNotCompiledException thrown correctly " + "when requesting Single Policy Updating");
				}
				
				// Now compile
				limid1.compile();
				
				// Create illegal orderings of different kinds
				
				// Two few nodes
				myOrdering.Clear();
				myOrdering.Add(d2_1);
				myOrdering.Add(d1_1);
				myOrdering.Add(d3_1);
				try
				{
					limid1.DefaultOrdering = myOrdering;
					return false;
				}
				catch (MLPInvalidNodeListException ex)
				{
					System.Console.Out.WriteLine("MLPInvalidNodeListException thrown correctly " + "when presenting too few nodes");
				}
				
				// Correct number but duplicates
				myOrdering.Clear();
				myOrdering.Add(d2_1);
				myOrdering.Add(d1_1);
				myOrdering.Add(d3_1);
				myOrdering.Add(d3_1);
				try
				{
					limid1.DefaultOrdering = myOrdering;
					return false;
				}
				catch (MLPInvalidNodeListException ex)
				{
					System.Console.Out.WriteLine("MLPInvalidNodeListException thrown correctly " + "when presenting two identical");
				}
				
				// A foreign node
				myOrdering.Clear();
				myOrdering.Add(d2_1);
				myOrdering.Add(d1_1);
				myOrdering.Add(d3_1);
				myOrdering.Add(d4_2);
				try
				{
					limid1.DefaultOrdering = myOrdering;
					return false;
				}
				catch (MLPInvalidNodeListException ex)
				{
					System.Console.Out.WriteLine("MLPInvalidNodeListException thrown correctly " + "when presenting foreign node");
				}
				
				// A utility node
				myOrdering.Clear();
				myOrdering.Add(d2_1);
				myOrdering.Add(d1_1);
				myOrdering.Add(d3_1);
				myOrdering.Add(u4_1);
				try
				{
					limid1.DefaultOrdering = myOrdering;
					return false;
				}
				catch (MLPInvalidNodeListException ex)
				{
					System.Console.Out.WriteLine("MLPInvalidNodeListException thrown correctly " + "when presenting utility node");
				}
				
				// A chance node
				myOrdering.Clear();
				myOrdering.Add(d2_1);
				myOrdering.Add(d1_1);
				myOrdering.Add(d3_1);
				myOrdering.Add(c4_1);
				try
				{
					limid1.DefaultOrdering = myOrdering;
					return false;
				}
				catch (MLPInvalidNodeListException ex)
				{
					System.Console.Out.WriteLine("MLPInvalidNodeListException thrown correctly " + "when presenting chance node");
				}
				
				LimidJunctionTree tree = limid1.JunctionTree;
				SupportClass.ListCollectionSupport cliques = tree.Cliques;
				
				LimidClique firstClique = (LimidClique) cliques[0];
				LimidClique secondClique = (LimidClique) cliques[1];
				
				// The two cliques must have different node sets:
				
				try
				{
					firstClique.Potential.divideByInSameDomain(secondClique.Potential);
					return false;
				}
				catch (MLPNotSameDomainException ex)
				{
					System.Console.Out.WriteLine("MLPNotSameDomainException thrown correctly");
				}
			}
			catch (MLPException ex)
			{
				// Something unexpected has happened
				return false;
			}
			
			
			return testResult;
		}
	}
}