using System;
using dk.esthauge.limid.gui;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.test
{
	
	/// <summary> <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// <p>Company: </p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	
	public class PigNet
	{
		
		/// <summary> Creates the pig breeder example from Lauritzen & Nilsson (2001). It also
		/// reads 2 Bayesian networks from files and shows all three models in a
		/// LimidFrame.
		/// </summary>
		/// <param name="args">No arguments should be given.
		/// </param>
		[STAThread]
		public static void  Main(System.String[] args)
		{
			try
			{
				LimidDomain domain = new LimidDomain("PigNet");
				
				BooleanLimidChanceNode[] h = new BooleanLimidChanceNode[4];
				BooleanLimidChanceNode[] t = new BooleanLimidChanceNode[3];
				BooleanDiscreteDecisionChanceNode[] d = new BooleanDiscreteDecisionChanceNode[3];
				TimeUtilityNode[] u = new TimeUtilityNode[4];
				int[] maximums;
				Potential table = new PotentialTable(domain.Nodes);
				for (int s = 0; s < 4; s++)
				{
					h[s] = new BooleanLimidChanceNode(domain);
					h[s].setPosition(new System.Drawing.Point(20 + 80 * s, 20));
					h[s].setStateLabel(0, "Healthy");
					h[s].setStateLabel(1, "Unhealthy");
					h[s].setName("h" + (s + 1));
					h[s].setLabel("h" + (s + 1));
					if (s == 0)
					{
						h[s].createTable();
						table = h[s].Table;
						table.setDataItem(0, 0.9);
						table.setDataItem(1, 0.1);
					}
					if (s > 0)
					{
						h[s].addParent(h[s - 1]);
						h[s].addParent(d[s - 1]);
						
						maximums = new int[3];
						maximums[1] = d[s - 1].getNumberOfStates();
						maximums[0] = h[s - 1].getNumberOfStates();
						maximums[2] = h[s].getNumberOfStates();
						table = h[s].Table;
						for (int j = 0; j < maximums[0]; j++)
							for (int i = 0; i < maximums[1]; i++)
								for (int k = 0; k < maximums[2]; k++)
								{
									int index = j * maximums[1] * maximums[2] + i * maximums[2] + k;
									if (i == 0)
									{
										if (j == 0)
										{
											if (k == 0)
												table.setDataItem(index, 0.8);
											else
												table.setDataItem(index, 0.2);
										}
										else
										{
											if (k == 0)
												table.setDataItem(index, 0.1);
											else
												table.setDataItem(index, 0.9);
										}
									}
									else
									{
										if (j == 0)
										{
											if (k == 0)
												table.setDataItem(index, 0.9);
											else
												table.setDataItem(index, 0.1);
										}
										else
										{
											if (k == 0)
												table.setDataItem(index, 0.5);
											else
												table.setDataItem(index, 0.5);
										}
									}
								}
					}
					if (s < 3)
					{
						t[s] = new BooleanLimidChanceNode(domain);
						t[s].addParent(h[s]);
						t[s].setPosition(new System.Drawing.Point(20 + 80 * s, 100));
						t[s].setName("t" + (s + 1));
						t[s].setLabel("t" + (s + 1));
						t[s].setStateLabel(0, "Negative");
						t[s].setStateLabel(1, "Positive");
						maximums = new int[2];
						maximums[0] = h[s].getNumberOfStates();
						maximums[1] = t[s].getNumberOfStates();
						table = t[s].Table;
						for (int i = 0; i < maximums[0]; i++)
							for (int j = 0; j < maximums[1]; j++)
							{
								int index = i * maximums[1] + j;
								if (i == 0)
								{
									if (j == 0)
										table.setDataItem(index, 0.9);
									else
										table.setDataItem(index, 0.1);
								}
								else
								{
									if (j == 0)
										table.setDataItem(index, 0.2);
									else
										table.setDataItem(index, 0.8);
								}
							}
						d[s] = new BooleanDiscreteDecisionChanceNode(domain);
						d[s].addParent(t[s]);
						d[s].setPosition(new System.Drawing.Point(40 + 80 * s, 180));
						d[s].setName("d" + (s + 1));
						d[s].setLabel("d" + (s + 1));
						d[s].setStateLabel(0, "Leave");
						d[s].setStateLabel(1, "Treat");
						
						u[s] = new TimeUtilityNode(domain);
						u[s].addParent(d[s]);
						u[s].setPosition(new System.Drawing.Point(20 + 80 * s, 260));
						u[s].setName("u" + (s + 1));
						u[s].setLabel("u" + (s + 1));
						table = u[s].Table;
						table.setDataItem(0, 0);
						table.setDataItem(1, - 100);
					}
					if (s == 3)
					{
						u[s] = new TimeUtilityNode(domain);
						u[s].addParent(h[s]);
						u[s].setPosition(new System.Drawing.Point(20 + 80 * 4, 20));
						u[s].setName("u" + (s + 1));
						u[s].setLabel("u" + (s + 1));
						table = u[s].Table;
						table.setDataItem(0, 1000);
						table.setDataItem(1, 300);
					}
				}
				
				/*
				// Transform to ID
				d[1].addParent(t[0]);
				d[1].addParent(d[0]);
				d[2].addParent(t[0]);
				d[2].addParent(t[1]);
				d[2].addParent(d[0]);
				d[2].addParent(d[1]);*/
				
				
				System.Console.Out.WriteLine("PigNet created");
				
				// Put labels equal to names
				NodeList nodes = domain.Nodes;
				System.Collections.IEnumerator itera = nodes.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (itera.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node node = (Node) itera.Current;
					node.setLabel(node.getName());
				}
				
				/*
				domain.compile();
				
				NodeList ordering = new NodeList();
				ordering.add(d[2]);
				ordering.add(d[1]);
				ordering.add(d[0]);
				
				domain.setDefaultOrdering(ordering);
				
				domain.singlePolicyUpdating();*/
				LimidFrame frame = new LimidFrame();
				frame.addLimid(domain);
				
				
				ModelReader mod = new ModelReader("C:/hugin/auto.xml");
				LimidDomain auto = mod.readLimid();
				frame.addLimid(auto);
				
				mod = new ModelReader("C:/hugin/alarm.xml");
				LimidDomain alarm = mod.readLimid();
				frame.addLimid(alarm);
				
				/*
				mod = new ModelReader("C:/hugin/svinelimid1.xml");
				LimidDomain svinMod = mod.readLimid();
				frame.addLimid(svinMod);*/
				//UPGRADE_TODO: System.Windows.Forms.Application.Run must be called to start a main form. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1135"'
				frame.Show();
				
				domain.toNetFile("C:/hugin/svinelimid.net");
				System.IO.FileStream fos = new System.IO.FileStream("C:/hugin/svinelimid.xml", System.IO.FileMode.Create);
				//UPGRADE_TODO: Class 'java.io.FileWriter' was converted to 'System.IO.StreamWriter' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073"'
				System.IO.StreamWriter fw = new System.IO.StreamWriter("C:/hugin/svinelimid.xml");
				domain.toStream("", fw);
				fw.Close();
			}
			catch (System.Exception eh)
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Throwable.getMessage' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				System.Console.Out.WriteLine(eh.Message);
				SupportClass.WriteStackTrace(eh, Console.Error);
			}
		}
	}
}