using System;
using dk.esthauge.limid.api;
namespace dk.esthauge.limid.test
{
	
	/// <summary> <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// <p>Company: </p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	
	public class RandomLimidGenerator
	{
		virtual public LimidDomain NextRandomLimid
		{
			get
			{
				LimidDomain limid = new LimidDomain("Random " + counter);
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				int nodeCount = (int) (SupportClass.Random.NextDouble() * maxNodes);
				if (nodeCount == 0)
					nodeCount = 1;
				int edgeCount = 0;
				try
				{
					for (int i = 0; i < nodeCount; i++)
					{
						Node node = new LabeledLimidChanceNode(limid);
						int xPos = (i % 10) * 90;
						int yPos = (i / 10) * 90;
						//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
						yPos = yPos + (int) (SupportClass.Random.NextDouble() * 30);
						//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
						xPos = xPos + (int) (SupportClass.Random.NextDouble() * 30);
						node.setPosition(new System.Drawing.Point(xPos, yPos));
					}
				}
				catch (MLPDuplicateNodeNameException ex)
				{
					SupportClass.WriteStackTrace(ex, Console.Error);
				}
				//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
				System.Collections.ArrayList nodes = SupportClass.CreateCollection(limid.Nodes);
				if (nodeCount > 1)
				{
					if (nodeCount == 2)
					{
						Node child = (Node) nodes[0];
						Node parent = (Node) nodes[1];
						try
						{
							child.addParent(parent);
						}
						catch (MLPOperationNotAllowedException ex)
						{
							SupportClass.WriteStackTrace(ex, Console.Error);
						}
					}
					// Add the same number of edges - one to each node
					else
						for (int i = 0; i < nodeCount; i++)
						{
							bool invalidEdge = true;
							int loops = 0;
							while (invalidEdge)
							{
								int nodeNumber = i;
								int parentNumber = 0;
								Node child = null;
								Node parent = null;
								try
								{
									// Don't try for ever...
									if (loops > nodeCount * 2)
									{
										limid = new LimidDomain("Gave up (1)");
										try
										{
											Node node = new LabeledLimidChanceNode(limid);
										}
										catch (System.Exception ex)
										{
											SupportClass.WriteStackTrace(ex, Console.Error);
										}
										counter++;
										return limid;
									}
									loops++;
									//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
									parentNumber = (int) (SupportClass.Random.NextDouble() * nodeCount);
									child = (Node) nodes[nodeNumber];
									parent = (Node) nodes[parentNumber];
									if (SupportClass.Random.NextDouble() > 0.5)
										child.addParent(parent);
									else
										parent.addParent(child);
									invalidEdge = false;
								}
								catch (MLPOperationNotAllowedException ex)
								{
									// Do nothing
								}
							}
						}
				}
				edgeCount = nodeCount;
				int loops = 0;
				// Continue adding edges until connected
				while (!limid.isConnected((Node) nodes[0]))
				{
					if (loops > 20)
					{
						limid = new LimidDomain("Gave up (2)");
						try
						{
							Node node = new LabeledLimidChanceNode(limid);
						}
						catch (System.Exception ex)
						{
							SupportClass.WriteStackTrace(ex, Console.Error);
						}
						counter++;
						return limid;
					}
					loops++;
					// 10 % more edges
					int additionalEdges = edgeCount / 10;
					if (additionalEdges == 0)
						additionalEdges = 1;
					edgeCount = edgeCount + additionalEdges;
					for (int i = 0; i < additionalEdges; i++)
					{
						bool invalidEdge = true;
						int loopsi = 0;
						while (invalidEdge)
						{
							try
							{
								// Don't try for ever...
								if (loopsi > nodeCount * 2)
								{
									limid = new LimidDomain("Gave up (3)");
									try
									{
										Node node = new LabeledLimidChanceNode(limid);
									}
									catch (System.Exception ex)
									{
										SupportClass.WriteStackTrace(ex, Console.Error);
									}
									counter++;
									return limid;
								}
								loopsi++;
								//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
								int nodeNumber = (int) (SupportClass.Random.NextDouble() * nodeCount);
								//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
								int parentNumber = (int) (SupportClass.Random.NextDouble() * nodeCount);
								Node child = (Node) nodes[nodeNumber];
								Node parent = (Node) nodes[parentNumber];
								child.addParent(parent);
								invalidEdge = false;
							}
							catch (MLPOperationNotAllowedException ex)
							{
								// Do nothing
							}
						}
					}
				}
				counter++;
				return limid;
			}
			
		}
		internal int counter = 0;
		internal int maxNodes;
		
		public RandomLimidGenerator(int maxNodes)
		{
			this.maxNodes = maxNodes;
		}
	}
}